"""
权重配置管理器 - 用于从外部配置文件加载并管理全局权重设置

此模块提供了从YAML或JSON配置文件中加载自定义权重设置的功能，
支持为特定羁绊和棋子设置自定义权重，并在整个应用程序中提供统一的权重配置。
"""
import os
import json
import yaml
import logging
from typing import Dict, Any, Optional

# 配置日志
logger = logging.getLogger("TFT-Strategist-WeightsConfig")

# 全局权重配置实例
_global_weights_config = None

class WeightsConfig:
    """
    权重配置管理器，用于从外部文件加载自定义权重设置
    """
    
    def __init__(self, config_path: Optional[str] = None):
        """
        初始化权重配置管理器
        
        Args:
            config_path: 配置文件路径，如果为None则使用默认路径
        """
        self.config_path = config_path or os.path.join("data", "weights_config.yaml")
        self.config_data = {}
        # 自动加载配置文件
        self.load_config()
    
    def load_config(self) -> Dict[str, Any]:
        """
        加载配置文件
        
        Returns:
            Dict[str, Any]: 包含配置数据的字典
        """
        if not os.path.exists(self.config_path):
            logger.warning(f"配置文件不存在: {self.config_path}")
            # 如果文件不存在，创建默认配置
            self._create_default_config()
            return self.config_data
        
        try:
            file_ext = os.path.splitext(self.config_path)[1].lower()
            
            if file_ext == '.json':
                with open(self.config_path, 'r', encoding='utf-8') as f:
                    self.config_data = json.load(f)
            elif file_ext in ['.yaml', '.yml']:
                with open(self.config_path, 'r', encoding='utf-8') as f:
                    self.config_data = yaml.safe_load(f)
            else:
                logger.error(f"不支持的配置文件格式: {file_ext}")
                self._create_default_config()
                return self.config_data
                
            logger.info(f"成功加载配置文件: {self.config_path}")
            return self.config_data
            
        except Exception as e:
            logger.error(f"加载配置文件失败: {str(e)}")
            # 如果加载失败，创建默认配置
            self._create_default_config()
            return self.config_data
    
    def get_synergy_weights(self) -> Dict[str, float]:
        """
        获取羁绊权重配置
        
        Returns:
            Dict[str, float]: 羁绊名称到权重的映射
        """
        return self.config_data.get('synergy_weights', {})
    
    def get_chess_weights(self) -> Dict[str, float]:
        """
        获取棋子权重配置
        
        Returns:
            Dict[str, float]: 棋子名称到权重的映射
        """
        return self.config_data.get('chess_weights', {})
    
    def get_base_weights(self) -> Dict[str, float]:
        """
        获取基础权重配置
        
        Returns:
            Dict[str, float]: 基础权重参数到权重值的映射
        """
        return self.config_data.get('base_weights', {})
    
    def get_synergy_level_weights(self) -> Dict[str, float]:
        """
        获取羁绊等级权重配置
        
        Returns:
            Dict[str, float]: 羁绊等级到权重的映射
        """
        return self.config_data.get('synergy_level_weights', {})
    
    def get_cost_weights(self) -> Dict[str, float]:
        """
        获取棋子费用权重配置
        
        Returns:
            Dict[str, float]: 棋子费用到权重的映射
        """
        return self.config_data.get('cost_weights', {})
    
    def set_synergy_weight(self, synergy_name: str, weight: float) -> None:
        """
        设置特定羁绊的权重
        
        Args:
            synergy_name: 羁绊名称
            weight: 权重值
        """
        if 'synergy_weights' not in self.config_data:
            self.config_data['synergy_weights'] = {}
        self.config_data['synergy_weights'][synergy_name] = weight
        self._save_config()
    
    def set_chess_weight(self, chess_name: str, weight: float) -> None:
        """
        设置特定棋子的权重
        
        Args:
            chess_name: 棋子名称
            weight: 权重值
        """
        if 'chess_weights' not in self.config_data:
            self.config_data['chess_weights'] = {}
        self.config_data['chess_weights'][chess_name] = weight
        self._save_config()
    
    def set_base_weight(self, param_name: str, weight: float) -> None:
        """
        设置基础权重参数
        
        Args:
            param_name: 参数名称
            weight: 权重值
        """
        if 'base_weights' not in self.config_data:
            self.config_data['base_weights'] = {}
        self.config_data['base_weights'][param_name] = weight
        self._save_config()
    
    def _save_config(self) -> None:
        """保存配置到文件"""
        try:
            os.makedirs(os.path.dirname(self.config_path), exist_ok=True)
            file_ext = os.path.splitext(self.config_path)[1].lower()
            
            if file_ext == '.json':
                with open(self.config_path, 'w', encoding='utf-8') as f:
                    json.dump(self.config_data, f, ensure_ascii=False, indent=2)
            elif file_ext in ['.yaml', '.yml']:
                with open(self.config_path, 'w', encoding='utf-8') as f:
                    yaml.dump(self.config_data, f, allow_unicode=True, default_flow_style=False)
            else:
                logger.error(f"不支持的配置文件格式: {file_ext}")
                return
                
            logger.info(f"成功保存配置文件: {self.config_path}")
            
        except Exception as e:
            logger.error(f"保存配置文件失败: {str(e)}")
    
    def _create_default_config(self) -> None:
        """创建默认配置文件"""
        default_config = {
            'base_weights': {
                'synergy_level_weight': 1.0,
                'synergy_count_weight': 0.5,
                'chess_cost_weight': 0.1
            },
            'synergy_weights': {
                '重装战士': 1.5,
                '魔法师': 1.2,
                '神谕者': 1.3,
                '斗士': 1.1
            },
            'chess_weights': {
                '亚索': 1.5,
                '艾希': 1.2,
                '璐璐': 1.3,
                '金克斯': 1.4
            },
            'synergy_level_weights': {
                '1': 1.0,
                '2': 1.2,
                '3': 1.5,
                '4': 1.8,
                '5': 2.0,
                '6': 2.3,
                '7': 2.6,
                '8': 3.0,
                '9': 3.3,
                '10': 3.5
            },
            'cost_weights': {
                '1': 1.0,
                '2': 1.2,
                '3': 1.5,
                '4': 1.8,
                '5': 2.0
            }
        }
        
        self.config_data = default_config
        self._save_config()


def get_global_weights_config(config_path: Optional[str] = None) -> WeightsConfig:
    """
    获取全局权重配置实例
    
    Args:
        config_path: 配置文件路径，如果为None则使用默认路径
        
    Returns:
        WeightsConfig: 全局权重配置实例
    """
    global _global_weights_config
    if _global_weights_config is None:
        _global_weights_config = WeightsConfig(config_path)
    return _global_weights_config 