"""
编程接口模块 - 提供程序化调用阵容推荐器的API

此模块实现了一组API函数，允许其他程序调用阵容推荐器的功能：
1. 获取游戏数据
2. 查询羁绊和棋子信息
3. 生成阵容推荐
4. 自定义评分系统
"""
from typing import Dict, List, Any, Optional, Union, Tuple
import logging
from dataclasses import asdict

from src.data_provider import DataQueryAPI, DataLoader
from src.recommendation import RecommendationEngine
from src.scoring import TeamScorer, ScoringConfig

# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger("TFT-Strategist-API")


class TFTStrategistAPI:
    """云顶之弈阵容推荐API类"""
    
    def __init__(self, use_local_data: bool = True):
        """
        初始化API
        
        Args:
            use_local_data: 是否使用本地数据，如果为False则从网络获取
        """
        self.data_loader = DataLoader(use_local_if_exists=use_local_data)
        self.api = DataQueryAPI(self.data_loader)
        self.recommendation_engine = RecommendationEngine(api=self.api)
        self.scorer = TeamScorer()
        logger.info(f"TFT-Strategist-API 已初始化，游戏版本: {self.get_version()}")
    
    def get_version(self) -> str:
        """
        获取当前游戏版本
        
        Returns:
            str: 游戏版本
        """
        return self.data_loader.get_latest_version() or "未知"
    
    def customize_scoring(self, **kwargs) -> None:
        """
        自定义评分系统参数
        
        Args:
            **kwargs: 评分参数，可以包括：
                synergy_level_weight: 羁绊等级权重
                synergy_count_weight: 羁绊数量权重
                chess_cost_weight: 棋子费用权重
                level_multipliers: 羁绊等级权重倍数字典
                cost_multipliers: 棋子费用权重倍数字典
        """
        self.scorer.customize_scoring(**kwargs)
        logger.info(f"评分系统参数已更新: {kwargs}")
    
    def get_scoring_config(self) -> Dict[str, Any]:
        """
        获取当前评分配置
        
        Returns:
            Dict[str, Any]: 评分配置字典
        """
        config = self.scorer.config
        return {
            'synergy_level_weight': config.synergy_level_weight,
            'synergy_count_weight': config.synergy_count_weight,
            'chess_cost_weight': config.chess_cost_weight,
            'level_multipliers': config.level_multipliers,
            'cost_multipliers': config.cost_multipliers
        }
    
    # 数据查询API
    
    def get_all_jobs(self) -> List[Dict[str, Any]]:
        """
        获取所有职业信息
        
        Returns:
            List[Dict[str, Any]]: 职业信息列表
        """
        return self.api.get_all_jobs()
    
    def get_all_races(self) -> List[Dict[str, Any]]:
        """
        获取所有特质信息
        
        Returns:
            List[Dict[str, Any]]: 特质信息列表
        """
        return self.api.get_all_races()
    
    def get_all_chess(self) -> List[Dict[str, Any]]:
        """
        获取所有棋子信息
        
        Returns:
            List[Dict[str, Any]]: 棋子信息列表
        """
        return self.api.get_all_chess()
    
    def get_job_by_name(self, name: str) -> Optional[Dict[str, Any]]:
        """
        根据名称获取职业信息
        
        Args:
            name: 职业名称
            
        Returns:
            Optional[Dict[str, Any]]: 职业信息，如果未找到则返回None
        """
        return self.api.get_job_by_name(name)
    
    def get_race_by_name(self, name: str) -> Optional[Dict[str, Any]]:
        """
        根据名称获取特质信息
        
        Args:
            name: 特质名称
            
        Returns:
            Optional[Dict[str, Any]]: 特质信息，如果未找到则返回None
        """
        return self.api.get_race_by_name(name)
    
    def get_chess_by_name(self, name: str) -> Optional[Dict[str, Any]]:
        """
        根据名称获取棋子信息
        
        Args:
            name: 棋子名称
            
        Returns:
            Optional[Dict[str, Any]]: 棋子信息，如果未找到则返回None
        """
        return self.api.get_chess_by_name(name)
    
    def get_chess_by_job(self, job_id: str) -> List[Dict[str, Any]]:
        """
        获取指定职业的所有棋子
        
        Args:
            job_id: 职业ID
            
        Returns:
            List[Dict[str, Any]]: 棋子信息列表
        """
        return self.api.get_chess_by_job(job_id)
    
    def get_chess_by_race(self, race_id: str) -> List[Dict[str, Any]]:
        """
        获取指定特质的所有棋子
        
        Args:
            race_id: 特质ID
            
        Returns:
            List[Dict[str, Any]]: 棋子信息列表
        """
        return self.api.get_chess_by_race(race_id)
    
    def get_synergies_of_chess(self, chess_id: str) -> List[Dict[str, Any]]:
        """
        获取棋子的所有羁绊信息
        
        Args:
            chess_id: 棋子ID
            
        Returns:
            List[Dict[str, Any]]: 羁绊信息列表
        """
        return self.api.get_synergies_of_chess(chess_id)
    
    def get_synergy_levels(self, synergy_id: str) -> Dict[str, str]:
        """
        获取羁绊的等级信息
        
        Args:
            synergy_id: 羁绊ID
            
        Returns:
            Dict[str, str]: 等级信息字典，键为等级，值为效果描述
        """
        return self.api.get_synergy_levels(synergy_id)
    
    # 阵容推荐API
    
    def recommend_team(
        self,
        population: int = 9,
        required_synergies: Optional[List[Dict[str, Any]]] = None,
        required_chess: Optional[List[Dict[str, Any]]] = None,
        max_results: int = 5
    ) -> List[Dict[str, Any]]:
        """
        推荐阵容
        
        Args:
            population: 阵容人口数，默认为9
            required_synergies: 必须包含的羁绊列表，每个羁绊为一个字典，包含name和level
            required_chess: 必须包含的棋子列表，每个棋子为一个字典，包含name
            max_results: 最多返回的推荐阵容数量
            
        Returns:
            List[Dict[str, Any]]: 推荐的阵容列表，每个阵容为一个字典
        """
        # 参数处理
        required_synergies = required_synergies or []
        required_chess = required_chess or []
        
        # 生成阵容推荐
        teams = self.recommendation_engine.recommend_team(
            population=population,
            required_synergies=required_synergies,
            required_chess=required_chess,
            max_results=max_results
        )
        
        # 转换为字典
        return [team.to_dict() for team in teams]
    
    def score_team(self, team_dict: Dict[str, Any]) -> float:
        """
        计算阵容评分
        
        Args:
            team_dict: 阵容字典，包含chess_list等字段
            
        Returns:
            float: 阵容评分
        """
        from src.recommendation.recommendation_engine import TeamComposition
        
        # 创建TeamComposition对象
        team = TeamComposition()
        
        # 添加棋子
        for chess in team_dict.get('chess_list', []):
            team.add_chess(chess)
        
        # 计算羁绊
        team.calculate_synergies(self.api)
        
        # 计算评分
        score = self.scorer.score_team(team)
        
        return score


# 全局API实例
_api_instance = None

def get_api(use_local_data: bool = True) -> TFTStrategistAPI:
    """
    获取API实例，单例模式
    
    Args:
        use_local_data: 是否使用本地数据
        
    Returns:
        TFTStrategistAPI: API实例
    """
    global _api_instance
    if _api_instance is None:
        _api_instance = TFTStrategistAPI(use_local_data=use_local_data)
    return _api_instance 