"""
命令行接口模块 - 提供用户与阵容推荐器交互的功能

此模块实现了一个命令行界面，允许用户以交互式方式：
1. 指定阵容人口
2. 添加必选羁绊和等级
3. 添加必选棋子
4. 获取阵容推荐结果并展示
"""
import argparse
import os
import sys
from typing import Dict, List, Any, Optional, Tuple
import logging

from src.data_provider import DataQueryAPI
from src.recommendation import RecommendationEngine
from src.scoring import TeamScorer, ScoringConfig
from src.config import get_global_weights_config

# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger("TFT-Strategist-CLI")


class TFTCommandLine:
    """云顶之弈阵容推荐命令行接口"""
    
    def __init__(self, config_path: Optional[str] = None):
        """
        初始化命令行接口
        
        Args:
            config_path: 配置文件路径，用于全局权重配置
        """
        # 加载全局权重配置
        self.weights_config = get_global_weights_config(config_path)
        
        # 初始化API和推荐引擎
        self.api = DataQueryAPI()
        self.scorer = TeamScorer(config_path=config_path)
        self.recommendation_engine = RecommendationEngine(api=self.api, scorer=self.scorer, config_path=config_path)
    
    def run(self, population=None, results=None, level_weight=None, count_weight=None, cost_weight=None):
        """
        运行命令行界面
        
        Args:
            population (int, optional): 阵容人口数量
            results (int, optional): 推荐结果数量
            level_weight (float, optional): 羁绊等级权重
            count_weight (float, optional): 羁绊数量权重
            cost_weight (float, optional): 棋子费用权重
        """
        parser = argparse.ArgumentParser(description='云顶之弈阵容推荐器命令行界面', add_help=False)
        parser.add_argument('--population', type=int, default=9, help='阵容人口数量，默认为9')
        parser.add_argument('--results', type=int, default=3, help='返回结果数量，默认为3')
        parser.add_argument('--level-weight', type=float, default=1.0, help='羁绊等级权重，默认为1.0')
        parser.add_argument('--count-weight', type=float, default=0.5, help='羁绊数量权重，默认为0.5')
        parser.add_argument('--cost-weight', type=float, default=0.1, help='棋子费用权重，默认为0.1')
        
        # 解析参数，但忽略未知的参数，以便与主程序的参数解析兼容
        args, _ = parser.parse_known_args()
        
        # 优先使用传入的参数，其次使用命令行参数
        population = population or args.population
        max_results = results or args.results
        level_weight = level_weight or args.level_weight
        count_weight = count_weight or args.count_weight
        cost_weight = cost_weight or args.cost_weight
        
        # 自定义评分权重，同时更新全局配置
        if level_weight is not None:
            self.weights_config.set_base_weight('synergy_level_weight', level_weight)
        if count_weight is not None:
            self.weights_config.set_base_weight('synergy_count_weight', count_weight)
        if cost_weight is not None:
            self.weights_config.set_base_weight('chess_cost_weight', cost_weight)
            
        # 更新评分系统的配置
        self.scorer.reload_config()
        
        # 初始化阵容参数
        required_synergies = []
        required_chess = []
        
        print("\n==== 云顶之弈阵容推荐器 ====\n")
        version = self.api.data_loader.get_latest_version()
        print(f"当前游戏版本: {version}")
        print(f"阵容人口: {population}")
        
        # 显示当前权重配置
        base_weights = self.weights_config.get_base_weights()
        print("\n当前评分权重配置:")
        print(f"  羁绊等级权重: {base_weights.get('synergy_level_weight', 1.0)}")
        print(f"  羁绊数量权重: {base_weights.get('synergy_count_weight', 0.5)}")
        print(f"  棋子费用权重: {base_weights.get('chess_cost_weight', 0.1)}")
        
        # 交互式添加必选羁绊
        print("\n-- 添加必选羁绊 --")
        while True:
            synergy_name = input("请输入羁绊名称（直接回车跳过）: ").strip()
            if not synergy_name:
                break
                
            synergy = self.api.get_synergy_by_name(synergy_name)
            if not synergy:
                print(f"未找到羁绊: {synergy_name}")
                continue
                
            synergy_id = synergy.get('jobId') or synergy.get('raceId')
            levels = self.api.get_synergy_levels(synergy_id)
            
            print(f"羁绊 {synergy_name} 的可用等级:")
            for level_str, effect in levels.items():
                print(f"  {level_str}: {effect}")
                
            while True:
                level_input = input("请输入所需等级（直接回车使用最低等级）: ").strip()
                if not level_input:
                    min_level = min(int(l) for l in levels.keys())
                    level = min_level
                    break
                    
                try:
                    level = int(level_input)
                    if str(level) in levels:
                        break
                    else:
                        print(f"无效的等级，请选择: {', '.join(levels.keys())}")
                except ValueError:
                    print("请输入数字")
            
            # 显示羁绊在配置中的权重
            synergy_weight = self.weights_config.get_synergy_weights().get(synergy_name, 1.0)
            print(f"羁绊 {synergy_name} 的当前权重为: {synergy_weight}")
            
            # 可选：调整羁绊权重
            adjust = input(f"是否调整该羁绊的权重？(y/n，当前: {synergy_weight}): ").strip().lower()
            if adjust == 'y':
                while True:
                    try:
                        new_weight = float(input(f"请输入新权重值 (当前: {synergy_weight}): ").strip())
                        self.weights_config.set_synergy_weight(synergy_name, new_weight)
                        print(f"已将羁绊 {synergy_name} 的权重设置为 {new_weight}")
                        break
                    except ValueError:
                        print("请输入有效的数字")
            
            required_synergies.append({
                'name': synergy_name,
                'level': level
            })
            print(f"已添加羁绊: {synergy_name}, 等级: {level}")
        
        # 交互式添加必选棋子
        print("\n-- 添加必选棋子 --")
        while True:
            chess_name = input("请输入棋子名称（直接回车跳过）: ").strip()
            if not chess_name:
                break
                
            chess = self.api.get_chess_by_name(chess_name)
            if not chess:
                print(f"未找到棋子: {chess_name}")
                continue
            
            # 显示棋子在配置中的权重
            chess_weight = self.weights_config.get_chess_weights().get(chess_name, 1.0)
            print(f"棋子 {chess_name} 的当前权重为: {chess_weight}")
            
            # 可选：调整棋子权重
            adjust = input(f"是否调整该棋子的权重？(y/n，当前: {chess_weight}): ").strip().lower()
            if adjust == 'y':
                while True:
                    try:
                        new_weight = float(input(f"请输入新权重值 (当前: {chess_weight}): ").strip())
                        self.weights_config.set_chess_weight(chess_name, new_weight)
                        print(f"已将棋子 {chess_name} 的权重设置为 {new_weight}")
                        break
                    except ValueError:
                        print("请输入有效的数字")
                
            required_chess.append({
                'name': chess_name
            })
            print(f"已添加棋子: {chess_name}")
        
        # 生成阵容推荐
        print("\n正在生成阵容推荐...")
        teams = self.recommendation_engine.recommend_team(
            population=population,
            required_synergies=required_synergies,
            required_chess=required_chess,
            max_results=max_results
        )
        
        # 显示推荐结果
        print("\n==== 阵容推荐结果 ====")
        for i, team in enumerate(teams, 1):
            print(f"\n*** 推荐阵容 #{i} (评分: {team.score:.2f}) ***")
            
            # 显示棋子列表
            print("棋子:")
            for chess in team.chess_list:
                # 获取棋子的羁绊
                job_names = []
                race_names = []
                
                for job_id in chess.get('jobIds', '').split(','):
                    if job_id:
                        job = self.api.get_job_by_id(job_id)
                        if job:
                            job_names.append(job['name'])
                
                for race_id in chess.get('raceIds', '').split(','):
                    if race_id:
                        race = self.api.get_race_by_id(race_id)
                        if race:
                            race_names.append(race['name'])
                
                job_str = ', '.join(job_names) if job_names else '无'
                race_str = ', '.join(race_names) if race_names else '无'
                
                print(f"  - {chess['displayName']} ({chess['price']}金币) | 职业: {job_str} | 特质: {race_str}")
            
            # 显示激活的羁绊
            print("\n激活的羁绊:")
            
            # 职业
            if team.synergy_levels['job']:
                print("职业:")
                for job_level in sorted(team.synergy_levels['job'], key=lambda x: x['name']):
                    print(f"  - {job_level['name']} ({job_level['count']}/{job_level['level']}): {job_level['effect']}")
            
            # 特质
            if team.synergy_levels['race']:
                print("特质:")
                for race_level in sorted(team.synergy_levels['race'], key=lambda x: x['name']):
                    print(f"  - {race_level['name']} ({race_level['count']}/{race_level['level']}): {race_level['effect']}")
            
            print("\n------------------------")


def main(population=None, results=None, level_weight=None, count_weight=None, cost_weight=None, config_path=None):
    """
    主函数
    
    Args:
        population (int, optional): 阵容人口数量
        results (int, optional): 推荐结果数量
        level_weight (float, optional): 羁绊等级权重
        count_weight (float, optional): 羁绊数量权重
        cost_weight (float, optional): 棋子费用权重
        config_path (str, optional): 配置文件路径
    """
    try:
        cli = TFTCommandLine(config_path=config_path)
        cli.run(population, results, level_weight, count_weight, cost_weight)
        return 0
    except KeyboardInterrupt:
        print("\n操作被用户中断")
        return 1
    except Exception as e:
        logger.error(f"发生错误: {str(e)}")
        return 1


if __name__ == "__main__":
    sys.exit(main()) 