"""
阵容推荐引擎 - 负责根据用户需求生成最优阵容

此模块实现了阵容推荐的核心逻辑，包括：
1. 基于羁绊、棋子和人口约束生成可行阵容
2. 根据评分标准对阵容进行排序
3. 返回最优阵容推荐结果
"""
from typing import Dict, List, Optional, Any, Set, Tuple, Union
import logging
import itertools
from dataclasses import dataclass, field
from src.data_provider import DataQueryAPI
from src.scoring.scoring_system import TeamScorer
from src.config import get_global_weights_config

# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger("TFT-Strategist-RecommendationEngine")

@dataclass
class TeamComposition:
    """表示一个云顶之弈阵容"""
    chess_list: List[Dict[str, Any]] = field(default_factory=list)
    synergy_counts: Dict[str, int] = field(default_factory=dict)
    synergy_levels: Dict[str, List[Dict[str, Any]]] = field(default_factory=dict)
    total_cost: int = 0
    score: float = 0.0
    
    @property
    def size(self) -> int:
        """阵容人口数"""
        return len(self.chess_list)
    
    def add_chess(self, chess: Dict[str, Any]) -> None:
        """添加棋子到阵容"""
        if chess not in self.chess_list:
            self.chess_list.append(chess)
            self.total_cost += int(chess.get('price', 0))
            # 更新羁绊计数
            
    def calculate_synergies(self, api: DataQueryAPI) -> None:
        """计算阵容的所有羁绊及其激活等级"""
        self.synergy_counts = {}
        self.synergy_levels = {'job': [], 'race': []}
        
        # 统计所有羁绊的数量
        for chess in self.chess_list:
            # 处理职业
            for job_id in chess.get('jobIds', '').split(','):
                if job_id:
                    self.synergy_counts[job_id] = self.synergy_counts.get(job_id, 0) + 1
            
            # 处理特质
            for race_id in chess.get('raceIds', '').split(','):
                if race_id:
                    self.synergy_counts[race_id] = self.synergy_counts.get(race_id, 0) + 1
        
        # 确定各羁绊激活的等级
        for synergy_id, count in self.synergy_counts.items():
            # 确保synergy_id是字符串
            synergy_id_str = str(synergy_id)
            synergy = api.get_synergy_by_id(synergy_id_str)
            if not synergy:
                logger.warning(f"未找到羁绊: {{'id': {synergy_id}}}")
                continue
            
            # 确定激活的等级
            levels = api.get_synergy_levels(synergy_id_str)
            active_levels = []
            
            for level_str, effect in levels.items():
                level = int(level_str)
                if count >= level:
                    active_level = {
                        'id': synergy_id_str,
                        'name': synergy.get('name', ''),
                        'level': level,
                        'count': count,
                        'effect': effect
                    }
                    active_levels.append(active_level)
            
            if active_levels:
                # 按等级排序
                active_levels.sort(key=lambda x: x['level'])
                # 将羁绊归类为职业或特质
                if 'jobId' in synergy:
                    self.synergy_levels['job'].extend(active_levels)
                else:
                    self.synergy_levels['race'].extend(active_levels)
    
    def to_dict(self) -> Dict[str, Any]:
        """将阵容转换为字典表示"""
        return {
            'chess_list': self.chess_list,
            'synergy_counts': self.synergy_counts,
            'synergy_levels': self.synergy_levels,
            'total_cost': self.total_cost,
            'size': self.size,
            'score': self.score
        }


class RecommendationEngine:
    """
    阵容推荐引擎，负责根据用户需求生成最优阵容
    """
    
    def __init__(self, api: DataQueryAPI, scorer: TeamScorer, config_path: Optional[str] = None, config_obj: Optional[Dict[str, Any]] = None):
        """
        初始化阵容推荐引擎
        
        Args:
            api: 数据查询API实例，如果为None则创建一个新的实例
            scorer: 阵容评分系统实例，如果为None则创建一个新的实例
            config_path: 配置文件路径，用于全局权重配置
            config_obj: 配置对象，优先于config_path使用
        """
        self.api = api if api else DataQueryAPI()
        
        # 加载全局权重配置
        if config_obj:
            self.weights_config = config_obj
        else:
            self.weights_config = get_global_weights_config(config_path).config_data
        
        # 创建阵容评分系统
        self.scorer = scorer if scorer else TeamScorer(config_path=config_path)
    
    def recommend_team(
        self,
        population: int = 9,
        required_synergies: Optional[List[Dict[str, Any]]] = None,
        required_chess: Optional[List[Dict[str, Any]]] = None,
        max_results: int = 5
    ) -> List[TeamComposition]:
        """
        根据用户需求推荐阵容
        
        Args:
            population: 阵容人口数，默认为9
            required_synergies: 必须包含的羁绊列表，每个羁绊为一个字典，包含id和最低激活等级
            required_chess: 必须包含的棋子列表
            max_results: 最多返回的推荐阵容数量
            
        Returns:
            List[TeamComposition]: 推荐的阵容列表，按评分从高到低排序
        """
        if population < 1 or population > 10:
            logger.warning(f"无效的人口数 {population}，已调整为默认值9")
            population = 9
        
        required_synergies = required_synergies or []
        required_chess = required_chess or []
        
        # 1. 处理必选棋子
        base_team = TeamComposition()
        for chess_info in required_chess:
            if isinstance(chess_info, dict) and 'name' in chess_info:
                chess = self.api.get_chess_by_name(chess_info['name'])
            elif isinstance(chess_info, dict) and 'id' in chess_info:
                chess = self.api.get_chess_by_id(str(chess_info['id']))
            else:
                chess = self.api.get_chess_by_name(str(chess_info))
            
            if chess:
                base_team.add_chess(chess)
            else:
                logger.warning(f"未找到棋子: {chess_info}")
        
        # 如果必选棋子已经超过了人口限制，则直接返回
        if base_team.size > population:
            logger.warning(f"必选棋子数量({base_team.size})超过了人口限制({population})")
            base_team.calculate_synergies(self.api)
            base_team.score = self.scorer.score_team(base_team)
            return [base_team]
        
        # 2. 获取羁绊所需的棋子集合
        synergy_chess_sets = []
        target_synergy_levels = {}
        
        for synergy_info in required_synergies:
            if isinstance(synergy_info, dict) and 'name' in synergy_info:
                synergy = self.api.get_synergy_by_name(synergy_info['name'])
                min_level = synergy_info.get('level', 1)
            elif isinstance(synergy_info, dict) and 'id' in synergy_info:
                synergy = self.api.get_synergy_by_id(str(synergy_info['id']))
                min_level = synergy_info.get('level', 1)
            else:
                synergy = self.api.get_synergy_by_name(str(synergy_info))
                min_level = 1
            
            if not synergy:
                logger.warning(f"未找到羁绊: {synergy_info}")
                continue
            
            synergy_id = synergy.get('jobId') or synergy.get('raceId')
            chess_list = self.api.get_chess_by_synergy(synergy_id)
            
            # 获取该羁绊的激活等级
            levels = self.api.get_synergy_levels(synergy_id)
            target_level = 0
            for level_str in levels.keys():
                level = int(level_str)
                if level >= min_level:
                    target_level = level
                    break
            
            if target_level == 0:
                logger.warning(f"羁绊 {synergy.get('name', '')} 找不到满足最低等级 {min_level} 的激活条件")
                continue
                
            # 考虑羁绊在全局配置中的权重
            weight = self.weights_config.get('synergy_weights', {}).get(synergy.get('name', ''), 1.0)
            
            synergy_chess_sets.append({
                'synergy': synergy,
                'chess_list': chess_list,
                'target_level': target_level,
                'weight': weight  # 添加权重信息
            })
            
            # 记录目标羁绊等级
            target_synergy_levels[synergy_id] = target_level
        
        # 3. 使用多种方法生成候选阵容
        logger.info("开始生成候选阵容...")
        candidate_teams = []
        
        # 3.1 首先使用传统方法生成阵容
        traditional_teams = self._generate_candidate_teams(base_team, synergy_chess_sets, population)
        if traditional_teams:
            logger.info(f"传统方法生成了 {len(traditional_teams)} 个候选阵容")
            candidate_teams.extend(traditional_teams)
        else:
            logger.info("传统方法未能生成任何阵容")
        
        # 3.2 使用贪婪算法生成阵容
        greedy_teams = self._generate_teams_greedy(base_team, population, target_synergy_levels)
        if greedy_teams:
            logger.info(f"贪婪算法生成了 {len(greedy_teams)} 个候选阵容")
            candidate_teams.extend(greedy_teams)
        
        # 3.3 使用穷举+剪枝算法生成阵容
        # 只在要求羁绊数不多时使用，避免搜索空间过大
        if len(target_synergy_levels) <= 4:
            exhaustive_teams = self._generate_teams_exhaustive(base_team, population, target_synergy_levels, max_teams=20)
            if exhaustive_teams:
                logger.info(f"穷举+剪枝算法生成了 {len(exhaustive_teams)} 个候选阵容")
                candidate_teams.extend(exhaustive_teams)
        
        # 3.4 如果仍然没有候选阵容，使用纯贪婪方法填充
        if not candidate_teams:
            logger.info("所有方法均未生成阵容，尝试使用贪婪填充")
            remaining_slots = population - base_team.size
            candidate_teams = self._fill_team_with_best_chess(base_team, remaining_slots, multiple_variants=True)
        
        # 移除重复阵容
        logger.info(f"总共生成了 {len(candidate_teams)} 个候选阵容，开始去重")
        unique_teams = []
        seen_chess_sets = set()
        
        for team in candidate_teams:
            chess_names = frozenset(chess.get('displayName', '') for chess in team.chess_list)
            if chess_names not in seen_chess_sets:
                seen_chess_sets.add(chess_names)
                unique_teams.append(team)
        
        candidate_teams = unique_teams
        logger.info(f"去重后剩余 {len(candidate_teams)} 个候选阵容")
        
        # 限制评分计算的阵容数量，避免性能问题
        max_teams_to_evaluate = min(2000, len(candidate_teams))
        teams_to_evaluate = candidate_teams[:max_teams_to_evaluate]
        
        # 4. 计算每个阵容的羁绊和评分
        logger.info(f"开始评分 {len(teams_to_evaluate)} 个候选阵容")
        
        # 计算羁绊和评分
        for team in teams_to_evaluate:
            team.calculate_synergies(self.api)
            team.score = self.scorer.score_team(team)
        
        # 按分数从高到低排序
        teams_to_evaluate.sort(key=lambda t: t.score, reverse=True)
        
        # 返回分数最高的几个阵容，并确保它们的差异性
        top_results = []
        seen_synergy_patterns = set()
        
        # 先添加评分最高的阵容
        if teams_to_evaluate:
            top_team = teams_to_evaluate[0]
            top_results.append(top_team)
            synergy_pattern = frozenset((synergy_id, count) for synergy_id, count in top_team.synergy_counts.items() if count >= 1)
            seen_synergy_patterns.add(synergy_pattern)
            teams_to_evaluate.pop(0)
        
        # 然后添加有足够差异性的其他高分阵容
        for team in teams_to_evaluate:
            if len(top_results) >= max_results:
                break
                
            # 计算羁绊模式，用于判断阵容差异性
            synergy_pattern = frozenset((synergy_id, count) for synergy_id, count in team.synergy_counts.items() if count >= 1)
            
            # 如果这个羁绊模式与已有结果差异足够大，则添加
            is_different = True
            for existing_pattern in seen_synergy_patterns:
                # 计算交集大小
                intersection = len(set(synergy_pattern).intersection(existing_pattern))
                union = len(set(synergy_pattern).union(existing_pattern))
                
                # Jaccard相似度
                if union > 0:
                    similarity = intersection / union
                    if similarity > 0.7:  # 70%相似度作为阈值
                        is_different = False
                        break
            
            if is_different:
                top_results.append(team)
                seen_synergy_patterns.add(synergy_pattern)
        
        # 如果找不到足够多有差异的阵容，返回评分最高的几个
        if len(top_results) < max_results and teams_to_evaluate:
            remaining = max_results - len(top_results)
            top_results.extend(teams_to_evaluate[:remaining])
        
        logger.info(f"最终推荐了 {len(top_results)} 个阵容")
        return top_results
    
    def _generate_candidate_teams(
        self,
        base_team: TeamComposition,
        synergy_chess_sets: List[Dict[str, Any]],
        population: int
    ) -> List[TeamComposition]:
        """
        根据羁绊要求生成候选阵容
        
        Args:
            base_team: 基础阵容，包含必选棋子
            synergy_chess_sets: 羁绊需要的棋子集合列表
            population: 人口限制
            
        Returns:
            List[TeamComposition]: 候选阵容列表
        """
        # 如果没有羁绊要求，则直接填充最佳棋子
        if not synergy_chess_sets:
            remaining_slots = population - base_team.size
            return self._fill_team_with_best_chess(base_team, remaining_slots)
        
        # 排序羁绊集合，根据权重值从高到低排序
        synergy_chess_sets.sort(key=lambda s: s.get('weight', 1.0), reverse=True)
        
        # 开始构建组合
        candidate_teams = []
        
        # 记录已经在基础阵容中的棋子
        base_chess_names = {chess.get('displayName') for chess in base_team.chess_list}
        
        # 1. 从每个羁绊中选择需要的棋子数量
        # 创建每个羁绊至少需要的棋子数组合
        min_chess_options = []
        for synergy_set in synergy_chess_sets:
            chess_list = synergy_set['chess_list']
            target_level = synergy_set['target_level']
            
            # 计算需要添加的棋子数量
            already_in_base = sum(1 for chess in base_team.chess_list
                                if any(job_id in chess.get('jobIds', '').split(',') 
                                    for job_id in [synergy_set['synergy'].get('jobId')])
                                or any(race_id in chess.get('raceIds', '').split(',')
                                    for race_id in [synergy_set['synergy'].get('raceId')]))
            
            needed_count = max(0, target_level - already_in_base)
            if needed_count <= 0:
                # 如果基础阵容已经满足该羁绊要求，则跳过
                continue
                
            # 从该羁绊的棋子中选择还未在基础阵容中的
            available_chess = [chess for chess in chess_list 
                            if chess.get('displayName') not in base_chess_names]
            
            # 如果可用棋子不足以满足羁绊要求
            if len(available_chess) < needed_count:
                # 不直接返回空列表，而是将这个羁绊标记为尽可能满足
                logger.warning(f"羁绊 {synergy_set['synergy'].get('name', '')} 可用棋子不足，尝试最大程度满足")
                if not available_chess:
                    continue
                needed_count = len(available_chess)
            
            # 所有可能的棋子组合
            chess_combinations = list(itertools.combinations(available_chess, needed_count))
            
            # 将权重因素纳入棋子选择决策
            weighted_combinations = []
            for combo in chess_combinations:
                # 计算组合的权重分数
                combo_weight = sum(self.weights_config.get('chess_weights', {}).get(chess.get('displayName', ''), 1.0) for chess in combo)
                weighted_combinations.append((combo, combo_weight))
            
            # 按权重从高到低排序
            weighted_combinations.sort(key=lambda x: x[1], reverse=True)
            
            # 选择权重较高的前N个组合（增加组合数量）
            max_combinations = min(20, len(weighted_combinations))
            top_combinations = [combo for combo, _ in weighted_combinations[:max_combinations]]
            min_chess_options.append(top_combinations)
        
        # 如果没有任何羁绊选项，则直接尝试填充最佳棋子
        if not min_chess_options:
            remaining_slots = population - base_team.size
            return self._fill_team_with_best_chess(base_team, remaining_slots)
        
        # 2. 尝试组合不同羁绊的棋子，生成可行阵容
        # 为每个羁绊选择一种棋子组合方式
        max_teams_to_generate = 1000  # 限制生成的候选阵容数量，避免组合爆炸
        teams_generated = 0
        
        for chess_selection in itertools.product(*min_chess_options):
            if teams_generated >= max_teams_to_generate:
                logger.info(f"已生成{teams_generated}个候选阵容，达到上限")
                break
                
            # 创建新的候选阵容
            candidate = TeamComposition()
            # 添加基础阵容的棋子
            for chess in base_team.chess_list:
                candidate.add_chess(chess)
            
            # 添加各羁绊选中的棋子
            all_selected_chess = set()
            for combo in chess_selection:
                for chess in combo:
                    if chess.get('displayName') not in all_selected_chess:
                        candidate.add_chess(chess)
                        all_selected_chess.add(chess.get('displayName'))
            
            # 检查是否超出人口限制
            if candidate.size <= population:
                # 如果有剩余人口，填充最佳棋子
                if candidate.size < population:
                    remaining_slots = population - candidate.size
                    filled_candidates = self._fill_team_with_best_chess(candidate, remaining_slots, multiple_variants=True)
                    candidate_teams.extend(filled_candidates)
                    teams_generated += len(filled_candidates)
                else:
                    candidate_teams.append(candidate)
                    teams_generated += 1
        
        # 如果没有生成任何候选阵容，则尝试放松某些羁绊的要求
        if not candidate_teams and min_chess_options:
            logger.info("尝试放松羁绊要求，生成更多候选阵容")
            # 尝试仅满足权重最高的部分羁绊
            priority_synergies = min_chess_options[:max(1, len(min_chess_options) // 2)]
            
            for chess_selection in itertools.product(*priority_synergies):
                if teams_generated >= max_teams_to_generate:
                    break
                    
                candidate = TeamComposition()
                for chess in base_team.chess_list:
                    candidate.add_chess(chess)
                
                all_selected_chess = set()
                for combo in chess_selection:
                    for chess in combo:
                        if chess.get('displayName') not in all_selected_chess:
                            candidate.add_chess(chess)
                            all_selected_chess.add(chess.get('displayName'))
                
                if candidate.size <= population:
                    if candidate.size < population:
                        remaining_slots = population - candidate.size
                        filled_candidates = self._fill_team_with_best_chess(candidate, remaining_slots, multiple_variants=True)
                        candidate_teams.extend(filled_candidates)
                        teams_generated += len(filled_candidates)
                    else:
                        candidate_teams.append(candidate)
                        teams_generated += 1
        
        # 如果仍然没有生成候选阵容，尝试只满足单个最高权重的羁绊
        if not candidate_teams and min_chess_options:
            logger.info("尝试只满足单个最高权重羁绊，生成更多候选阵容")
            highest_priority_synergy = min_chess_options[0]
            
            for combo in highest_priority_synergy:
                candidate = TeamComposition()
                for chess in base_team.chess_list:
                    candidate.add_chess(chess)
                
                for chess in combo:
                    candidate.add_chess(chess)
                
                if candidate.size <= population:
                    if candidate.size < population:
                        remaining_slots = population - candidate.size
                        filled_candidates = self._fill_team_with_best_chess(candidate, remaining_slots, multiple_variants=True)
                        candidate_teams.extend(filled_candidates)
                    else:
                        candidate_teams.append(candidate)
        
        # 如果经过所有尝试仍然没有生成候选阵容，使用基础阵容加上填充
        if not candidate_teams:
            logger.info("无法满足任何羁绊要求，直接填充最佳棋子")
            remaining_slots = population - base_team.size
            return self._fill_team_with_best_chess(base_team, remaining_slots, multiple_variants=True)
            
        return candidate_teams
    
    def _fill_team_with_best_chess(
        self,
        base_team: TeamComposition,
        remaining_slots: int,
        multiple_variants: bool = False
    ) -> List[TeamComposition]:
        """
        用最佳棋子填充剩余槽位
        
        Args:
            base_team: 基础阵容
            remaining_slots: 剩余槽位数量
            multiple_variants: 是否生成多个变种阵容
            
        Returns:
            List[TeamComposition]: 填充后的阵容列表
        """
        if remaining_slots <= 0:
            return [base_team]
            
        # 计算每个棋子的羁绊价值
        all_chess = self.api.get_all_chess()
        base_chess_names = {chess.get('displayName') for chess in base_team.chess_list}
        
        # 排除已在基础阵容中的棋子
        available_chess = [chess for chess in all_chess 
                        if chess.get('displayName') not in base_chess_names]
        
        # 计算基础阵容中已有的羁绊
        base_synergies = {}
        for chess in base_team.chess_list:
            # 职业
            for job_id in chess.get('jobIds', '').split(','):
                if job_id:
                    base_synergies[job_id] = base_synergies.get(job_id, 0) + 1
            
            # 特质
            for race_id in chess.get('raceIds', '').split(','):
                if race_id:
                    base_synergies[race_id] = base_synergies.get(race_id, 0) + 1
        
        # 评估每个可用棋子的价值
        chess_values = []
        for chess in available_chess:
            value = 0
            
            # 根据费用评估基础价值
            cost = int(chess.get('price', 1))
            cost_multiplier = float(self.weights_config.get('cost_weights', {}).get(str(cost), 1.0))
            value += cost * cost_multiplier
            
            # 根据羁绊评估额外价值
            synergy_contribution = 0
            potential_new_level = False
            
            for job_id in chess.get('jobIds', '').split(','):
                if job_id:
                    # 如果能与基础阵容形成羁绊，增加价值
                    synergy = self.api.get_synergy_by_id(job_id)
                    if not synergy:
                        continue
                        
                    if job_id in base_synergies:
                        # 检查是否能激活新的羁绊等级
                        levels = self.api.get_synergy_levels(job_id)
                        current_count = base_synergies[job_id]
                        
                        for level_str in sorted(levels.keys()):
                            level = int(level_str)
                            if current_count + 1 >= level > current_count:
                                # 如果添加这个棋子后能激活新的等级
                                weight = self.weights_config.get('synergy_weights', {}).get(synergy.get('name', ''), 1.0)
                                level_multiplier = float(self.weights_config.get('synergy_level_weights', {}).get(level_str, 1.0))
                                synergy_contribution += weight * level_multiplier * 10
                                potential_new_level = True
                                break
                        
                        # 即使不能激活新等级，也增加一些价值
                        synergy_contribution += 2
                    else:
                        # 新的羁绊，给予一些价值
                        weight = self.weights_config.get('synergy_weights', {}).get(synergy.get('name', ''), 1.0)
                        synergy_contribution += weight * 3
            
            for race_id in chess.get('raceIds', '').split(','):
                if race_id:
                    # 如果能与基础阵容形成羁绊，增加价值
                    synergy = self.api.get_synergy_by_id(race_id)
                    if not synergy:
                        continue
                        
                    if race_id in base_synergies:
                        # 检查是否能激活新的羁绊等级
                        levels = self.api.get_synergy_levels(race_id)
                        current_count = base_synergies[race_id]
                        
                        for level_str in sorted(levels.keys()):
                            level = int(level_str)
                            if current_count + 1 >= level > current_count:
                                # 如果添加这个棋子后能激活新的等级
                                weight = self.weights_config.get('synergy_weights', {}).get(synergy.get('name', ''), 1.0)
                                level_multiplier = float(self.weights_config.get('synergy_level_weights', {}).get(level_str, 1.0))
                                synergy_contribution += weight * level_multiplier * 10
                                potential_new_level = True
                                break
                        
                        # 即使不能激活新等级，也增加一些价值
                        synergy_contribution += 2
                    else:
                        # 新的羁绊，给予一些价值
                        weight = self.weights_config.get('synergy_weights', {}).get(synergy.get('name', ''), 1.0)
                        synergy_contribution += weight * 3
            
            # 如果棋子能激活新的羁绊等级，给予额外加成
            if potential_new_level:
                value += synergy_contribution * 1.5
            else:
                value += synergy_contribution
            
            # 添加棋子自定义权重
            chess_weight = self.weights_config.get('chess_weights', {}).get(chess.get('displayName', ''), 1.0)
            value *= chess_weight
            
            chess_values.append((chess, value))
        
        # 按价值从高到低排序
        chess_values.sort(key=lambda x: x[1], reverse=True)
        
        if not multiple_variants:
            # 单一变种模式：创建一个最优阵容
            candidate = TeamComposition()
            # 添加基础阵容的棋子
            for chess in base_team.chess_list:
                candidate.add_chess(chess)
            
            # 添加价值最高的剩余槽位数量的棋子
            for i in range(min(remaining_slots, len(chess_values))):
                candidate.add_chess(chess_values[i][0])
            
            return [candidate]
        else:
            # 多变种模式：创建多个不同的阵容组合
            candidates = []
            
            # 1. 添加基于纯价值排序的标准阵容
            standard_candidate = TeamComposition()
            for chess in base_team.chess_list:
                standard_candidate.add_chess(chess)
                
            for i in range(min(remaining_slots, len(chess_values))):
                standard_candidate.add_chess(chess_values[i][0])
            
            candidates.append(standard_candidate)
            
            # 如果剩余槽位较少或棋子不足，就不生成更多变种
            if remaining_slots <= 1 or len(chess_values) < remaining_slots * 2:
                return candidates
            
            # 2. 添加基于不同羁绊侧重的变种阵容
            # 识别基础阵容中最有潜力的羁绊
            potential_synergies = {}
            for synergy_id, count in base_synergies.items():
                # 获取该羁绊的信息
                synergy = self.api.get_synergy_by_id(synergy_id)
                if not synergy:
                    continue
                
                # 获取该羁绊的等级
                levels = self.api.get_synergy_levels(synergy_id)
                next_level = None
                
                # 寻找下一个可能的等级
                for level_str in sorted(levels.keys(), key=int):
                    level = int(level_str)
                    if level > count:
                        next_level = level
                        break
                
                if next_level is not None:
                    # 计算距离下一级需要的棋子数
                    needed = next_level - count
                    # 如果在剩余槽位内可以实现，则视为潜力羁绊
                    if needed <= remaining_slots:
                        synergy_name = synergy.get('name', '')
                        weight = self.weights_config.get('synergy_weights', {}).get(synergy_name, 1.0)
                        potential_synergies[synergy_id] = {
                            'name': synergy_name,
                            'needed': needed,
                            'weight': weight,
                            'next_level': next_level
                        }
            
            # 按权重和距离下一级的需求排序
            sorted_potentials = sorted(
                potential_synergies.items(),
                key=lambda x: (x[1]['weight'] / max(1, x[1]['needed']), -x[1]['needed']),
                reverse=True
            )
            
            # 为每个高潜力羁绊创建一个变种
            max_variants = min(5, len(sorted_potentials))
            for i in range(min(max_variants, len(sorted_potentials))):
                synergy_id, synergy_info = sorted_potentials[i]
                
                # 寻找能提供该羁绊的棋子
                synergy_chess = []
                for chess, _ in chess_values:
                    if (synergy_id in chess.get('jobIds', '').split(',') or 
                        synergy_id in chess.get('raceIds', '').split(',')):
                        synergy_chess.append(chess)
                
                if len(synergy_chess) >= synergy_info['needed']:
                    # 创建新的变种阵容
                    variant = TeamComposition()
                    for chess in base_team.chess_list:
                        variant.add_chess(chess)
                    
                    # 添加强化特定羁绊的棋子
                    for j in range(min(synergy_info['needed'], len(synergy_chess))):
                        variant.add_chess(synergy_chess[j])
                    
                    # 如果还有剩余槽位，填充价值最高的其他棋子
                    remaining = remaining_slots - synergy_info['needed']
                    if remaining > 0:
                        # 排除已添加的棋子
                        added_chess_names = {chess.get('displayName') for chess in variant.chess_list}
                        for chess, _ in chess_values:
                            if (chess.get('displayName') not in added_chess_names and 
                                remaining > 0):
                                variant.add_chess(chess)
                                remaining -= 1
                    
                    candidates.append(variant)
            
            # 3. 基于费用分布的变种
            if remaining_slots >= 3:  # 只有在有足够槽位时才考虑费用分布
                # 高费用变种
                high_cost_variant = TeamComposition()
                for chess in base_team.chess_list:
                    high_cost_variant.add_chess(chess)
                
                # 按费用从高到低排序
                cost_sorted_chess = sorted(
                    available_chess,
                    key=lambda c: (int(c.get('price', 0)), 
                                  sum(self.weights_config.get('chess_weights', {}).get(c.get('displayName', ''), 1.0) for c in base_team.chess_list)),
                    reverse=True
                )
                
                for i in range(min(remaining_slots, len(cost_sorted_chess))):
                    high_cost_variant.add_chess(cost_sorted_chess[i])
                
                candidates.append(high_cost_variant)
            
            return candidates

    def _generate_teams_greedy(
        self,
        base_team: TeamComposition,
        population: int,
        all_synergy_levels: Optional[Dict[str, int]] = None
    ) -> List[TeamComposition]:
        """
        使用贪婪算法生成阵容，尝试激活尽可能多的高价值羁绊
        
        Args:
            base_team: 基础阵容
            population: 人口上限
            all_synergy_levels: 尝试激活的羁绊及其等级，格式为 {synergy_id: min_level}
            
        Returns:
            List[TeamComposition]: 生成的阵容列表
        """
        logger.info("使用贪婪算法生成阵容")
        
        # 获取所有可用棋子
        all_chess = self.api.get_all_chess()
        base_chess_names = {chess.get('displayName') for chess in base_team.chess_list}
        available_chess = [chess for chess in all_chess 
                          if chess.get('displayName') not in base_chess_names]
        
        # 计算基础阵容中已有的羁绊
        base_synergies = {}
        for chess in base_team.chess_list:
            # 职业
            for job_id in chess.get('jobIds', '').split(','):
                if job_id:
                    base_synergies[job_id] = base_synergies.get(job_id, 0) + 1
            
            # 特质
            for race_id in chess.get('raceIds', '').split(','):
                if race_id:
                    base_synergies[race_id] = base_synergies.get(race_id, 0) + 1
        
        # 如果未指定羁绊等级，则计算当前阵容中所有羁绊到下一级的距离
        if not all_synergy_levels:
            all_synergy_levels = {}
            
            # 检查当前羁绊
            for synergy_id, count in base_synergies.items():
                levels = self.api.get_synergy_levels(synergy_id)
                next_level = None
                
                # 寻找下一个等级
                for level_str in sorted(levels.keys(), key=int):
                    level = int(level_str)
                    if level > count:
                        next_level = level
                        break
                
                if next_level is not None:
                    all_synergy_levels[synergy_id] = next_level
            
            # 检查其他可能的羁绊
            all_synergies = self.api.get_all_synergies()
            for synergy in all_synergies:
                synergy_id = synergy.get('jobId') or synergy.get('raceId')
                if synergy_id and synergy_id not in all_synergy_levels:
                    levels = self.api.get_synergy_levels(synergy_id)
                    if levels:
                        # 取最低等级
                        lowest_level = min(int(lvl) for lvl in levels.keys())
                        all_synergy_levels[synergy_id] = lowest_level
        
        # 为每个羁绊分配权重
        synergy_weights = {}
        for synergy_id, target_level in all_synergy_levels.items():
            synergy = self.api.get_synergy_by_id(synergy_id)
            if not synergy:
                continue
                
            name = synergy.get('name', '')
            weight = self.weights_config.get('synergy_weights', {}).get(name, 1.0)
            level_multiplier = self.weights_config.get('synergy_level_weights', {}).get(str(target_level), 1.0)
            
            # 计算激活该羁绊所需的成本（还需要的棋子数）
            current_count = base_synergies.get(synergy_id, 0)
            needed = max(0, target_level - current_count)
            
            # 根据权重、等级和成本计算综合价值
            if needed > 0:
                # 考虑难度系数，需求越多越难
                difficulty_factor = min(1.0, 3 / needed) if needed > 3 else 1.0
                value = weight * level_multiplier * difficulty_factor * 10
                synergy_weights[synergy_id] = (value, needed, target_level)
        
        # 按价值从高到低排序
        sorted_synergies = sorted(
            synergy_weights.items(),
            key=lambda x: x[1][0],
            reverse=True
        )
        
        # 生成多个候选阵容，每个从不同角度构建
        candidates = []
        
        # 1. 从高价值羁绊开始构建
        for start_idx in range(min(5, len(sorted_synergies))):
            # 创建新阵容
            team = TeamComposition()
            for chess in base_team.chess_list:
                team.add_chess(chess)
            
            # 获取当前阵容中的棋子
            current_chess_names = {chess.get('displayName') for chess in team.chess_list}
            
            # 获取当前激活的羁绊
            current_synergies = base_synergies.copy()
            
            # 从某个高价值羁绊开始
            focused_synergies = sorted_synergies[start_idx:] + sorted_synergies[:start_idx]
            
            # 尝试添加棋子，直到达到人口限制
            remaining_slots = population - team.size
            
            while remaining_slots > 0 and focused_synergies:
                synergy_id, (_, needed, target_level) = focused_synergies[0]
                focused_synergies = focused_synergies[1:]
                
                # 获取当前羁绊的等级
                current_level = current_synergies.get(synergy_id, 0)
                still_needed = max(0, target_level - current_level)
                
                if still_needed <= 0:
                    continue
                
                # 找出所有可以提供该羁绊的棋子
                synergy_chess = []
                for chess in available_chess:
                    if (synergy_id in chess.get('jobIds', '').split(',') or 
                        synergy_id in chess.get('raceIds', '').split(',')):
                        if chess.get('displayName') not in current_chess_names:
                            synergy_chess.append(chess)
                
                # 按价值和费用排序（优先低费用）
                synergy_chess.sort(key=lambda c: (
                    self.weights_config.get('chess_weights', {}).get(c.get('displayName', ''), 1.0),
                    -int(c.get('price', 0))
                ), reverse=True)
                
                # 添加所需的棋子
                added = 0
                for chess in synergy_chess:
                    if remaining_slots <= 0:
                        break
                    
                    if added >= still_needed:
                        break
                    
                    # 添加棋子
                    team.add_chess(chess)
                    current_chess_names.add(chess.get('displayName'))
                    remaining_slots -= 1
                    added += 1
                    
                    # 更新羁绊计数
                    for job_id in chess.get('jobIds', '').split(','):
                        if job_id:
                            current_synergies[job_id] = current_synergies.get(job_id, 0) + 1
                    
                    for race_id in chess.get('raceIds', '').split(','):
                        if race_id:
                            current_synergies[race_id] = current_synergies.get(race_id, 0) + 1
            
            # 如果还有剩余槽位，填充价值最高的棋子
            if remaining_slots > 0:
                # 评估剩余棋子
                available = [c for c in available_chess if c.get('displayName') not in current_chess_names]
                
                # 计算每个棋子的价值
                chess_values = []
                for chess in available:
                    value = 0
                    
                    # 基础价值
                    cost = int(chess.get('price', 1))
                    cost_multiplier = float(self.weights_config.get('cost_weights', {}).get(str(cost), 1.0))
                    value += cost * cost_multiplier
                    
                    # 羁绊价值
                    for job_id in chess.get('jobIds', '').split(','):
                        if job_id and job_id in current_synergies:
                            synergy = self.api.get_synergy_by_id(job_id)
                            if synergy:
                                levels = self.api.get_synergy_levels(job_id)
                                current = current_synergies[job_id]
                                
                                # 检查是否能激活新等级
                                for level_str in sorted(levels.keys()):
                                    level = int(level_str)
                                    if current + 1 >= level > current:
                                        value += 20
                                        break
                                value += 2
                    
                    for race_id in chess.get('raceIds', '').split(','):
                        if race_id and race_id in current_synergies:
                            synergy = self.api.get_synergy_by_id(race_id)
                            if synergy:
                                levels = self.api.get_synergy_levels(race_id)
                                current = current_synergies[race_id]
                                
                                # 检查是否能激活新等级
                                for level_str in sorted(levels.keys()):
                                    level = int(level_str)
                                    if current + 1 >= level > current:
                                        value += 20
                                        break
                                value += 2
                    
                    # 棋子权重
                    chess_weight = self.weights_config.get('chess_weights', {}).get(chess.get('displayName', ''), 1.0)
                    value *= chess_weight
                    
                    chess_values.append((chess, value))
                
                # 排序并添加
                chess_values.sort(key=lambda x: x[1], reverse=True)
                
                for i in range(min(remaining_slots, len(chess_values))):
                    team.add_chess(chess_values[i][0])
            
            candidates.append(team)
        
        # 2. 基于羁绊组合的方式构建阵容
        # 选择最有潜力的几个羁绊组合
        potential_synergy_combinations = []
        
        # 获取所有潜在的羁绊对
        synergy_pairs = []
        for i in range(len(sorted_synergies)):
            for j in range(i+1, len(sorted_synergies)):
                syn1_id, (val1, needed1, _) = sorted_synergies[i]
                syn2_id, (val2, needed2, _) = sorted_synergies[j]
                
                # 如果两个羁绊一起的需求不超过人口上限
                if needed1 + needed2 <= population - base_team.size:
                    total_value = val1 + val2
                    synergy_pairs.append((syn1_id, syn2_id, total_value))
        
        # 为每个有潜力的羁绊对构建阵容
        synergy_pairs.sort(key=lambda x: x[2], reverse=True)
        
        for pair_idx in range(min(5, len(synergy_pairs))):
            syn1_id, syn2_id, _ = synergy_pairs[pair_idx]
            
            # 创建新阵容
            team = TeamComposition()
            for chess in base_team.chess_list:
                team.add_chess(chess)
            
            # 获取当前阵容中的棋子
            current_chess_names = {chess.get('displayName') for chess in team.chess_list}
            
            # 获取当前激活的羁绊
            current_synergies = base_synergies.copy()
            
            # 先处理这两个羁绊
            priority_synergies = [
                (syn1_id, synergy_weights[syn1_id]),
                (syn2_id, synergy_weights[syn2_id])
            ]
            
            # 剩余的羁绊
            other_synergies = [s for s in sorted_synergies if s[0] not in {syn1_id, syn2_id}]
            
            # 所有羁绊顺序
            focused_synergies = priority_synergies + other_synergies
            
            # 尝试添加棋子，直到达到人口限制
            remaining_slots = population - team.size
            
            while remaining_slots > 0 and focused_synergies:
                synergy_id, (_, needed, target_level) = focused_synergies[0]
                focused_synergies = focused_synergies[1:]
                
                # 获取当前羁绊的等级
                current_level = current_synergies.get(synergy_id, 0)
                still_needed = max(0, target_level - current_level)
                
                if still_needed <= 0:
                    continue
                
                # 找出所有可以提供该羁绊的棋子
                synergy_chess = []
                for chess in available_chess:
                    if (synergy_id in chess.get('jobIds', '').split(',') or 
                        synergy_id in chess.get('raceIds', '').split(',')):
                        if chess.get('displayName') not in current_chess_names:
                            synergy_chess.append(chess)
                
                # 按价值和费用排序
                synergy_chess.sort(key=lambda c: (
                    self.weights_config.get('chess_weights', {}).get(c.get('displayName', ''), 1.0),
                    -int(c.get('price', 0))
                ), reverse=True)
                
                # 添加所需的棋子
                added = 0
                for chess in synergy_chess:
                    if remaining_slots <= 0:
                        break
                    
                    if added >= still_needed:
                        break
                    
                    # 添加棋子
                    team.add_chess(chess)
                    current_chess_names.add(chess.get('displayName'))
                    remaining_slots -= 1
                    added += 1
                    
                    # 更新羁绊计数
                    for job_id in chess.get('jobIds', '').split(','):
                        if job_id:
                            current_synergies[job_id] = current_synergies.get(job_id, 0) + 1
                    
                    for race_id in chess.get('raceIds', '').split(','):
                        if race_id:
                            current_synergies[race_id] = current_synergies.get(race_id, 0) + 1
            
            # 如果还有剩余槽位，填充价值最高的棋子
            if remaining_slots > 0:
                # 评估剩余棋子
                available = [c for c in available_chess if c.get('displayName') not in current_chess_names]
                
                # 计算每个棋子的价值
                chess_values = []
                for chess in available:
                    value = 0
                    
                    # 基础价值
                    cost = int(chess.get('price', 1))
                    cost_multiplier = float(self.weights_config.get('cost_weights', {}).get(str(cost), 1.0))
                    value += cost * cost_multiplier
                    
                    # 羁绊价值
                    for job_id in chess.get('jobIds', '').split(','):
                        if job_id and job_id in current_synergies:
                            synergy = self.api.get_synergy_by_id(job_id)
                            if synergy:
                                levels = self.api.get_synergy_levels(job_id)
                                current = current_synergies[job_id]
                                
                                # 检查是否能激活新等级
                                for level_str in sorted(levels.keys()):
                                    level = int(level_str)
                                    if current + 1 >= level > current:
                                        value += 20
                                        break
                                value += 2
                    
                    for race_id in chess.get('raceIds', '').split(','):
                        if race_id and race_id in current_synergies:
                            synergy = self.api.get_synergy_by_id(race_id)
                            if synergy:
                                levels = self.api.get_synergy_levels(race_id)
                                current = current_synergies[race_id]
                                
                                # 检查是否能激活新等级
                                for level_str in sorted(levels.keys()):
                                    level = int(level_str)
                                    if current + 1 >= level > current:
                                        value += 20
                                        break
                                value += 2
                    
                    # 棋子权重
                    chess_weight = self.weights_config.get('chess_weights', {}).get(chess.get('displayName', ''), 1.0)
                    value *= chess_weight
                    
                    chess_values.append((chess, value))
                
                # 排序并添加
                chess_values.sort(key=lambda x: x[1], reverse=True)
                
                for i in range(min(remaining_slots, len(chess_values))):
                    team.add_chess(chess_values[i][0])
            
            candidates.append(team)
            
        return candidates 

    def _generate_teams_exhaustive(
        self,
        base_team: TeamComposition,
        population: int,
        target_synergies: Optional[Dict[str, int]] = None,
        max_teams: int = 20
    ) -> List[TeamComposition]:
        """
        使用穷举+剪枝算法生成阵容，尝试各种组合并保留最佳结果
        
        Args:
            base_team: 基础阵容
            population: 人口上限
            target_synergies: 目标羁绊及等级，格式为 {synergy_id: min_level}
            max_teams: 最多生成的阵容数量
            
        Returns:
            List[TeamComposition]: 生成的阵容列表
        """
        logger.info("使用穷举+剪枝算法生成阵容")
        
        # 获取所有可用棋子
        all_chess = self.api.get_all_chess()
        base_chess_names = {chess.get('displayName') for chess in base_team.chess_list}
        available_chess = [chess for chess in all_chess 
                          if chess.get('displayName') not in base_chess_names]
        
        # 计算基础阵容中已有的羁绊
        base_synergies = {}
        for chess in base_team.chess_list:
            # 职业
            for job_id in chess.get('jobIds', '').split(','):
                if job_id:
                    base_synergies[job_id] = base_synergies.get(job_id, 0) + 1
            
            # 特质
            for race_id in chess.get('raceIds', '').split(','):
                if race_id:
                    base_synergies[race_id] = base_synergies.get(race_id, 0) + 1
        
        # 如果未指定目标羁绊，使用所有羁绊的最低等级作为目标
        if not target_synergies:
            target_synergies = {}
            
            # 先考虑已有羁绊的提升
            for synergy_id, count in base_synergies.items():
                levels = self.api.get_synergy_levels(synergy_id)
                next_level = None
                
                for level_str in sorted(levels.keys(), key=int):
                    level = int(level_str)
                    if level > count:
                        next_level = level
                        break
                
                if next_level is not None:
                    target_synergies[synergy_id] = next_level
            
            # 再考虑其他羁绊
            if not target_synergies:
                all_synergies = self.api.get_all_synergies()
                for synergy in all_synergies:
                    synergy_id = synergy.get('jobId') or synergy.get('raceId')
                    if synergy_id and synergy_id not in target_synergies:
                        levels = self.api.get_synergy_levels(synergy_id)
                        if levels:
                            # 取最低等级
                            lowest_level = min(int(lvl) for lvl in levels.keys())
                            target_synergies[synergy_id] = lowest_level
        
        # 对棋子进行价值评估
        chess_values = []
        for chess in available_chess:
            # 基础价值
            value = 0
            cost = int(chess.get('price', 1))
            cost_multiplier = float(self.weights_config.get('cost_weights', {}).get(str(cost), 1.0))
            value += cost * cost_multiplier
            
            # 羁绊价值
            for job_id in chess.get('jobIds', '').split(','):
                if job_id:
                    if job_id in target_synergies:
                        # 如果是目标羁绊，增加高价值
                        synergy = self.api.get_synergy_by_id(job_id)
                        if synergy:
                            name = synergy.get('name', '')
                            weight = self.weights_config.get('synergy_weights', {}).get(name, 1.0)
                            value += weight * 10
                    elif job_id in base_synergies:
                        # 如果是已有羁绊，增加一些价值
                        value += 5
            
            for race_id in chess.get('raceIds', '').split(','):
                if race_id:
                    if race_id in target_synergies:
                        # 如果是目标羁绊，增加高价值
                        synergy = self.api.get_synergy_by_id(race_id)
                        if synergy:
                            name = synergy.get('name', '')
                            weight = self.weights_config.get('synergy_weights', {}).get(name, 1.0)
                            value += weight * 10
                    elif race_id in base_synergies:
                        # 如果是已有羁绊，增加一些价值
                        value += 5
            
            # 棋子权重
            chess_weight = self.weights_config.get('chess_weights', {}).get(chess.get('displayName', ''), 1.0)
            value *= chess_weight
            
            chess_values.append((chess, value))
        
        # 按价值从高到低排序
        chess_values.sort(key=lambda x: x[1], reverse=True)
        top_chess = [chess for chess, _ in chess_values[:min(20, len(chess_values))]]
        
        # 剩余需要添加的棋子数量
        remaining_slots = population - base_team.size
        
        # 如果剩余槽位较小，可以直接穷举所有组合
        if remaining_slots <= 4 and len(top_chess) <= 15:
            logger.info(f"剩余槽位较少，进行完全穷举 (槽位: {remaining_slots}, 棋子: {len(top_chess)})")
            return self._exhaustive_search(base_team, top_chess, remaining_slots, target_synergies, max_teams)
        
        # 否则使用贪婪搜索 + 部分穷举
        logger.info(f"剩余槽位较多，使用贪婪搜索 + 部分穷举 (槽位: {remaining_slots})")
        
        # 生成候选阵容列表
        candidate_teams = []
        
        # 1. 先使用贪婪算法生成一些候选阵容
        greedy_teams = []
        
        # 基本贪婪阵容
        basic_team = TeamComposition()
        for chess in base_team.chess_list:
            basic_team.add_chess(chess)
        
        # 添加价值最高的棋子
        for i in range(min(remaining_slots, len(top_chess))):
            basic_team.add_chess(top_chess[i])
        
        greedy_teams.append(basic_team)
        
        # 为每个目标羁绊生成一个侧重阵容
        for synergy_id, target_level in target_synergies.items():
            # 计算当前等级
            current_level = base_synergies.get(synergy_id, 0)
            needed = max(0, target_level - current_level)
            
            if needed <= 0 or needed > remaining_slots:
                continue
            
            # 找出能提供该羁绊的棋子
            synergy_chess = []
            for chess in available_chess:
                if (synergy_id in chess.get('jobIds', '').split(',') or 
                    synergy_id in chess.get('raceIds', '').split(',')):
                    synergy_chess.append(chess)
            
            if len(synergy_chess) < needed:
                continue
            
            # 按价值排序
            synergy_chess.sort(key=lambda c: next((v for ch, v in chess_values if ch.get('displayName') == c.get('displayName')), 0), reverse=True)
            
            # 创建新阵容
            team = TeamComposition()
            for chess in base_team.chess_list:
                team.add_chess(chess)
            
            # 添加羁绊棋子
            for i in range(min(needed, len(synergy_chess))):
                team.add_chess(synergy_chess[i])
            
            # 如果还有剩余槽位，填充价值最高的棋子
            remaining = remaining_slots - needed
            if remaining > 0:
                added_chess = {chess.get('displayName') for chess in team.chess_list}
                
                for chess, _ in chess_values:
                    if remaining <= 0:
                        break
                    
                    if chess.get('displayName') not in added_chess:
                        team.add_chess(chess)
                        remaining -= 1
            
            greedy_teams.append(team)
        
        candidate_teams.extend(greedy_teams)
        
        # 2. 针对部分关键位置进行穷举搜索
        # 如果剩余槽位较多，可以固定一部分高价值棋子，对剩余槽位进行穷举
        if remaining_slots > 4:
            # 固定一部分高价值棋子
            fixed_count = remaining_slots - 4  # 留出4个槽位进行穷举
            
            # 创建基础阵容
            partial_base = TeamComposition()
            for chess in base_team.chess_list:
                partial_base.add_chess(chess)
            
            # 添加固定的高价值棋子
            for i in range(min(fixed_count, len(top_chess))):
                partial_base.add_chess(top_chess[i])
            
            # 剩余可选棋子
            added_chess = {chess.get('displayName') for chess in partial_base.chess_list}
            remaining_chess = [chess for chess in top_chess if chess.get('displayName') not in added_chess]
            
            # 进行局部穷举
            partial_exhaustive = self._exhaustive_search(partial_base, remaining_chess, remaining_slots - fixed_count, target_synergies, max_teams // 2)
            candidate_teams.extend(partial_exhaustive)
        
        # 3. 随机生成一些阵容以增加多样性
        import random
        random_teams = []
        
        for _ in range(5):  # 生成5个随机阵容
            random_team = TeamComposition()
            for chess in base_team.chess_list:
                random_team.add_chess(chess)
            
            added_chess = {chess.get('displayName') for chess in random_team.chess_list}
            random_selection = random.sample(available_chess, min(remaining_slots, len(available_chess)))
            
            for chess in random_selection:
                if chess.get('displayName') not in added_chess:
                    random_team.add_chess(chess)
            
            random_teams.append(random_team)
        
        candidate_teams.extend(random_teams)
        
        # 确保结果不超过最大阵容数
        return candidate_teams[:max_teams]
    
    def _exhaustive_search(
        self,
        base_team: TeamComposition,
        available_chess: List[Dict[str, Any]],
        slots: int,
        target_synergies: Dict[str, int],
        max_teams: int
    ) -> List[TeamComposition]:
        """
        针对给定的基础阵容和可用棋子，穷举所有可能的组合
        
        Args:
            base_team: 基础阵容
            available_chess: 可用的棋子列表
            slots: 剩余槽位数
            target_synergies: 目标羁绊及等级
            max_teams: 最多返回的阵容数量
            
        Returns:
            List[TeamComposition]: 生成的阵容列表
        """
        # 如果可用棋子太多，进行剪枝
        if len(available_chess) > 12 and slots < 5:
            # 按价值评估缩小棋子池
            chess_values = []
            for chess in available_chess:
                value = 0
                
                # 检查是否能激活目标羁绊
                for job_id in chess.get('jobIds', '').split(','):
                    if job_id and job_id in target_synergies:
                        value += 10
                
                for race_id in chess.get('raceIds', '').split(','):
                    if race_id and race_id in target_synergies:
                        value += 10
                
                # 基础价值
                cost = int(chess.get('price', 1))
                value += cost
                
                # 棋子权重
                chess_weight = self.weights_config.get('chess_weights', {}).get(chess.get('displayName', ''), 1.0)
                value *= chess_weight
                
                chess_values.append((chess, value))
            
            # 按价值从高到低排序
            chess_values.sort(key=lambda x: x[1], reverse=True)
            
            # 取前N个
            available_chess = [chess for chess, _ in chess_values[:min(12, len(chess_values))]]
        
        # 计算基础阵容中已有的羁绊
        base_synergies = {}
        for chess in base_team.chess_list:
            # 职业
            for job_id in chess.get('jobIds', '').split(','):
                if job_id:
                    base_synergies[job_id] = base_synergies.get(job_id, 0) + 1
            
            # 特质
            for race_id in chess.get('raceIds', '').split(','):
                if race_id:
                    base_synergies[race_id] = base_synergies.get(race_id, 0) + 1
        
        # 生成所有可能的组合
        candidate_teams = []
        
        # 如果槽位数为0，直接返回基础阵容
        if slots == 0:
            return [base_team]
        
        # 生成所有slots大小的组合
        import itertools
        combinations = list(itertools.combinations(available_chess, min(slots, len(available_chess))))
        
        # 如果组合太多，进行采样
        max_combinations = 1000
        if len(combinations) > max_combinations:
            import random
            combinations = random.sample(combinations, max_combinations)
        
        # 用各种组合生成阵容
        for combo in combinations:
            team = TeamComposition()
            
            # 添加基础阵容
            for chess in base_team.chess_list:
                team.add_chess(chess)
            
            # 添加组合中的棋子
            for chess in combo:
                team.add_chess(chess)
            
            # 计算羁绊
            synergy_counts = base_synergies.copy()
            
            for chess in combo:
                # 职业
                for job_id in chess.get('jobIds', '').split(','):
                    if job_id:
                        synergy_counts[job_id] = synergy_counts.get(job_id, 0) + 1
                
                # 特质
                for race_id in chess.get('raceIds', '').split(','):
                    if race_id:
                        synergy_counts[race_id] = synergy_counts.get(race_id, 0) + 1
            
            # 计算能达到的目标羁绊数量
            achieved_targets = 0
            for synergy_id, target_level in target_synergies.items():
                if synergy_counts.get(synergy_id, 0) >= target_level:
                    achieved_targets += 1
            
            # 存储信息供后续排序使用
            team.score = achieved_targets
            candidate_teams.append(team)
        
        # 按达成的目标数量从高到低排序
        candidate_teams.sort(key=lambda t: t.score, reverse=True)
        
        # 取前max_teams个结果
        return candidate_teams[:max_teams] 