"""
阵容推荐模块演示脚本
"""
from src.data_provider import DataQueryAPI
from src.recommendation import RecommendationEngine
from src.scoring import TeamScorer, ScoringConfig
from src.config import get_global_weights_config
from typing import Optional


def main(config_path: Optional[str] = None):
    """
    主函数
    
    Args:
        config_path: 配置文件路径，用于全局权重配置
    """
    print("=== 云顶之弈阵容推荐模块演示 ===")
    
    # 初始化数据查询API
    print("\n正在加载数据...")
    api = DataQueryAPI()
    
    # 获取全局权重配置
    weights_config = get_global_weights_config(config_path)
    
    # 初始化评分系统和推荐引擎
    scorer = TeamScorer(config_path=config_path)
    engine = RecommendationEngine(api=api, scorer=scorer, config_path=config_path)
    
    # 显示数据基本信息
    print(f"\n当前游戏版本: {api.data_loader.get_latest_version()}")
    print(f"职业数量: {len(api.get_all_jobs())}")
    print(f"特质数量: {len(api.get_all_races())}")
    print(f"棋子数量: {len(api.get_all_chess())}")
    
    # 显示当前权重配置
    base_weights = weights_config.get_base_weights()
    print("\n当前评分权重配置:")
    print(f"  羁绊等级权重: {base_weights.get('synergy_level_weight', 1.0)}")
    print(f"  羁绊数量权重: {base_weights.get('synergy_count_weight', 0.5)}")
    print(f"  棋子费用权重: {base_weights.get('chess_cost_weight', 0.1)}")
    
    # 互动推荐示例
    while True:
        print("\n请选择推荐方式:")
        print("1. 指定人口数量")
        print("2. 指定必选羁绊")
        print("3. 指定必选棋子")
        print("4. 综合条件推荐")
        print("5. 调整权重配置")
        print("6. 退出")
        
        choice = input("请输入选项 (1-6): ")
        
        if choice == '1':
            try:
                population = int(input("请输入阵容人口数 (1-10): "))
                if population < 1 or population > 10:
                    print("无效的人口数，请输入1-10之间的整数")
                    continue
                
                print(f"\n正在生成{population}人口的最佳阵容...")
                teams = engine.recommend_team(population=population)
                _display_recommended_teams(teams)
            except ValueError:
                print("请输入有效的整数")
        
        elif choice == '2':
            synergy_name = input("请输入羁绊名称: ")
            synergy = api.get_synergy_by_name(synergy_name)
            if not synergy:
                print(f"未找到名为 '{synergy_name}' 的羁绊")
                continue
            
            # 显示羁绊在配置中的权重
            synergy_weight = weights_config.get_synergy_weights().get(synergy_name, 1.0)
            print(f"羁绊 {synergy_name} 的当前权重为: {synergy_weight}")
            
            # 可选：调整羁绊权重
            adjust = input(f"是否调整该羁绊的权重？(y/n，当前: {synergy_weight}): ").strip().lower()
            if adjust == 'y':
                try:
                    new_weight = float(input(f"请输入新权重值 (当前: {synergy_weight}): ").strip())
                    weights_config.set_synergy_weight(synergy_name, new_weight)
                    print(f"已将羁绊 {synergy_name} 的权重设置为 {new_weight}")
                except ValueError:
                    print("请输入有效的数字")
            
            try:
                level = int(input(f"请输入 {synergy_name} 的目标等级 (默认为1): ") or "1")
                required_synergies = [{'name': synergy_name, 'level': level}]
                
                population = int(input("请输入阵容人口数 (1-10，默认为9): ") or "9")
                if population < 1 or population > 10:
                    print("无效的人口数，已调整为默认值9")
                    population = 9
                
                print(f"\n正在生成包含 {synergy_name} ({level}级) 的{population}人口阵容...")
                teams = engine.recommend_team(
                    population=population,
                    required_synergies=required_synergies
                )
                _display_recommended_teams(teams)
            except ValueError:
                print("请输入有效的整数")
        
        elif choice == '3':
            chess_name = input("请输入棋子名称: ")
            chess = api.get_chess_by_name(chess_name)
            if not chess:
                print(f"未找到名为 '{chess_name}' 的棋子")
                continue
            
            # 显示棋子在配置中的权重
            chess_weight = weights_config.get_chess_weights().get(chess_name, 1.0)
            print(f"棋子 {chess_name} 的当前权重为: {chess_weight}")
            
            # 可选：调整棋子权重
            adjust = input(f"是否调整该棋子的权重？(y/n，当前: {chess_weight}): ").strip().lower()
            if adjust == 'y':
                try:
                    new_weight = float(input(f"请输入新权重值 (当前: {chess_weight}): ").strip())
                    weights_config.set_chess_weight(chess_name, new_weight)
                    print(f"已将棋子 {chess_name} 的权重设置为 {new_weight}")
                except ValueError:
                    print("请输入有效的数字")
            
            try:
                population = int(input("请输入阵容人口数 (1-10，默认为9): ") or "9")
                if population < 1 or population > 10:
                    print("无效的人口数，已调整为默认值9")
                    population = 9
                
                required_chess = [{'name': chess_name}]
                
                print(f"\n正在生成包含 {chess_name} 的{population}人口阵容...")
                teams = engine.recommend_team(
                    population=population,
                    required_chess=required_chess
                )
                _display_recommended_teams(teams)
            except ValueError:
                print("请输入有效的整数")
        
        elif choice == '4':
            try:
                # 收集必选羁绊
                required_synergies = []
                while True:
                    synergy_name = input("请输入必选羁绊名称 (留空结束): ")
                    if not synergy_name:
                        break
                    
                    synergy = api.get_synergy_by_name(synergy_name)
                    if not synergy:
                        print(f"未找到名为 '{synergy_name}' 的羁绊")
                        continue
                    
                    # 显示羁绊在配置中的权重
                    synergy_weight = weights_config.get_synergy_weights().get(synergy_name, 1.0)
                    print(f"羁绊 {synergy_name} 的当前权重为: {synergy_weight}")
                    
                    # 可选：调整羁绊权重
                    adjust = input(f"是否调整该羁绊的权重？(y/n，当前: {synergy_weight}): ").strip().lower()
                    if adjust == 'y':
                        try:
                            new_weight = float(input(f"请输入新权重值 (当前: {synergy_weight}): ").strip())
                            weights_config.set_synergy_weight(synergy_name, new_weight)
                            print(f"已将羁绊 {synergy_name} 的权重设置为 {new_weight}")
                        except ValueError:
                            print("请输入有效的数字")
                    
                    level = int(input(f"请输入 {synergy_name} 的目标等级 (默认为1): ") or "1")
                    required_synergies.append({'name': synergy_name, 'level': level})
                
                # 收集必选棋子
                required_chess = []
                while True:
                    chess_name = input("请输入必选棋子名称 (留空结束): ")
                    if not chess_name:
                        break
                    
                    chess = api.get_chess_by_name(chess_name)
                    if not chess:
                        print(f"未找到名为 '{chess_name}' 的棋子")
                        continue
                    
                    # 显示棋子在配置中的权重
                    chess_weight = weights_config.get_chess_weights().get(chess_name, 1.0)
                    print(f"棋子 {chess_name} 的当前权重为: {chess_weight}")
                    
                    # 可选：调整棋子权重
                    adjust = input(f"是否调整该棋子的权重？(y/n，当前: {chess_weight}): ").strip().lower()
                    if adjust == 'y':
                        try:
                            new_weight = float(input(f"请输入新权重值 (当前: {chess_weight}): ").strip())
                            weights_config.set_chess_weight(chess_name, new_weight)
                            print(f"已将棋子 {chess_name} 的权重设置为 {new_weight}")
                        except ValueError:
                            print("请输入有效的数字")
                    
                    required_chess.append({'name': chess_name})
                
                # 设置人口
                population = int(input("请输入阵容人口数 (1-10，默认为9): ") or "9")
                if population < 1 or population > 10:
                    print("无效的人口数，已调整为默认值9")
                    population = 9
                
                # 生成推荐阵容
                print("\n正在生成符合条件的阵容...")
                teams = engine.recommend_team(
                    population=population,
                    required_synergies=required_synergies,
                    required_chess=required_chess
                )
                _display_recommended_teams(teams)
            except ValueError:
                print("请输入有效的整数")
        
        elif choice == '5':
            print("\n=== 调整权重配置 ===")
            print("1. 调整基础权重")
            print("2. 调整羁绊权重")
            print("3. 调整棋子权重")
            print("4. 返回主菜单")
            
            sub_choice = input("请输入选项 (1-4): ")
            
            if sub_choice == '1':
                try:
                    print("\n当前基础权重:")
                    base_weights = weights_config.get_base_weights()
                    for key, value in base_weights.items():
                        print(f"  {key}: {value}")
                    
                    param = input("\n请输入要调整的参数名称 (例如: synergy_level_weight): ")
                    if param not in base_weights:
                        print(f"参数 '{param}' 不存在")
                        continue
                    
                    new_value = float(input(f"请输入新的权重值 (当前: {base_weights.get(param, 1.0)}): "))
                    weights_config.set_base_weight(param, new_value)
                    print(f"已将 {param} 的权重设置为 {new_value}")
                    
                    # 更新评分系统
                    scorer.reload_config()
                except ValueError:
                    print("请输入有效的数字")
            
            elif sub_choice == '2':
                synergy_name = input("请输入羁绊名称: ")
                synergy = api.get_synergy_by_name(synergy_name)
                if not synergy:
                    print(f"未找到名为 '{synergy_name}' 的羁绊")
                    continue
                
                try:
                    current_weight = weights_config.get_synergy_weights().get(synergy_name, 1.0)
                    new_weight = float(input(f"请输入新的权重值 (当前: {current_weight}): "))
                    weights_config.set_synergy_weight(synergy_name, new_weight)
                    print(f"已将羁绊 {synergy_name} 的权重设置为 {new_weight}")
                except ValueError:
                    print("请输入有效的数字")
            
            elif sub_choice == '3':
                chess_name = input("请输入棋子名称: ")
                chess = api.get_chess_by_name(chess_name)
                if not chess:
                    print(f"未找到名为 '{chess_name}' 的棋子")
                    continue
                
                try:
                    current_weight = weights_config.get_chess_weights().get(chess_name, 1.0)
                    new_weight = float(input(f"请输入新的权重值 (当前: {current_weight}): "))
                    weights_config.set_chess_weight(chess_name, new_weight)
                    print(f"已将棋子 {chess_name} 的权重设置为 {new_weight}")
                except ValueError:
                    print("请输入有效的数字")
            
            elif sub_choice == '4':
                continue
                
            else:
                print("无效选项，请重新输入")
                
        elif choice == '6':
            break
        
        else:
            print("无效选项，请重新输入")


def _display_recommended_teams(teams):
    """显示推荐的阵容"""
    if not teams:
        print("未找到符合条件的阵容")
        return
    
    print(f"\n找到 {len(teams)} 个推荐阵容:")
    for i, team in enumerate(teams, 1):
        print(f"\n=== 推荐阵容 {i} (评分: {team.score:.2f}) ===")
        
        # 显示激活的羁绊
        print("\n激活的羁绊:")
        # 按照等级从高到低排序
        all_synergies = []
        for category in ['job', 'race']:
            for synergy in team.synergy_levels.get(category, []):
                all_synergies.append(synergy)
        
        all_synergies.sort(key=lambda x: (x['count'], x['level']), reverse=True)
        
        for synergy in all_synergies:
            print(f"  - {synergy['name']} ({synergy['count']}/{synergy['level']}): {synergy['effect']}")
        
        # 显示棋子列表
        print("\n棋子列表:")
        # 按照费用从高到低排序
        sorted_chess = sorted(team.chess_list, key=lambda x: int(x.get('price', '1')), reverse=True)
        for chess in sorted_chess:
            price = chess.get('price', '未知')
            # 获取棋子的羁绊
            job_ids = chess.get('jobIds', '').split(',')
            race_ids = chess.get('raceIds', '').split(',')
            
            job_names = []
            for job_id in job_ids:
                if job_id:
                    job = next((j for j in team.synergy_levels.get('job', []) if j['id'] == job_id), None)
                    if job:
                        job_names.append(job['name'])
            
            race_names = []
            for race_id in race_ids:
                if race_id:
                    race = next((r for r in team.synergy_levels.get('race', []) if r['id'] == race_id), None)
                    if race:
                        race_names.append(race['name'])
            
            job_str = ', '.join(job_names) if job_names else '无'
            race_str = ', '.join(race_names) if race_names else '无'
            
            print(f"  - {chess['displayName']} ({price}金币) | 职业: {job_str} | 特质: {race_str}")


if __name__ == "__main__":
    main() 