"""
阵容评分系统 - 负责对阵容进行综合评分

此模块提供了对云顶之弈阵容的评分功能，考虑多个因素：
1. 羁绊数量及等级
2. 羁绊协同性
3. 棋子费用分布
4. 阵容整体强度
5. 自定义羁绊和棋子权重
"""
from typing import Dict, List, Any, Optional
import logging
from dataclasses import dataclass
import os
from src.config import get_global_weights_config
from src.data_provider import DataQueryAPI

# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger("TFT-Strategist-ScoringSystem")

@dataclass
class ScoringConfig:
    """阵容评分配置"""
    # 基础权重
    synergy_level_weight: float = 1.0  # 羁绊等级权重
    synergy_count_weight: float = 0.5  # 羁绊种类数量权重
    chess_cost_weight: float = 0.1     # 棋子费用权重
    
    # 羁绊等级权重倍数
    level_multipliers: Dict[int, float] = None
    
    # 棋子费用权重倍数
    cost_multipliers: Dict[str, float] = None
    
    # 自定义羁绊权重
    synergy_weights: Dict[str, float] = None
    
    # 自定义棋子权重
    chess_weights: Dict[str, float] = None
    
    def __post_init__(self):
        """初始化默认值"""
        if self.level_multipliers is None:
            # 羁绊等级越高，权重越大
            self.level_multipliers = {
                1: 1.0,
                2: 1.5,
                3: 2.0,
                4: 3.0,
                5: 4.0,
                6: 5.0,
                7: 6.0,
                8: 7.0,
                9: 8.0
            }
        
        if self.cost_multipliers is None:
            # 棋子费用越高，权重越大
            self.cost_multipliers = {
                '1': 1.0,
                '2': 1.5,
                '3': 2.0,
                '4': 3.0,
                '5': 4.0
            }
            
        # 初始化自定义权重为空字典
        if self.synergy_weights is None:
            self.synergy_weights = {}
            
        if self.chess_weights is None:
            self.chess_weights = {}


class TeamScorer:
    """
    阵容评分系统，负责对阵容进行综合评分
    """
    
    def __init__(self, api: Optional[DataQueryAPI] = None, config_path: Optional[str] = None, config_obj: Optional[Dict[str, Any]] = None):
        """
        初始化阵容评分系统
        
        Args:
            api: 数据查询API
            config_path: 外部配置文件路径，如果提供则从中加载自定义权重
            config_obj: 配置对象，优先于config_path使用
        """
        self.api = api if api else DataQueryAPI()
        
        # 加载权重配置
        if config_obj:
            self.weights_config = config_obj
        else:
            self.weights_config = get_global_weights_config(config_path).config_data
        
        # 加载基础权重参数
        self.synergy_level_weight = self.weights_config.get('base_weights', {}).get('synergy_level_weight', 1.0)
        self.synergy_count_weight = self.weights_config.get('base_weights', {}).get('synergy_count_weight', 0.5)
        self.chess_cost_weight = self.weights_config.get('base_weights', {}).get('chess_cost_weight', 0.1)
        
        # 加载自定义权重
        self.synergy_weights = self.weights_config.get('synergy_weights', {})
        self.chess_weights = self.weights_config.get('chess_weights', {})
        
        # 加载羁绊等级权重和棋子费用权重倍数
        synergy_level_weights = self.weights_config.get('synergy_level_weights', {})
        cost_weights = self.weights_config.get('cost_weights', {})
        
        # 将键转换为正确的类型
        self.level_multipliers = {int(k) if isinstance(k, str) else k: float(v) for k, v in synergy_level_weights.items()}
        self.cost_multipliers = {k: float(v) for k, v in cost_weights.items()}
    
    def reload_config(self) -> None:
        """重新加载配置"""
        if isinstance(self.weights_config, dict):
            # 针对直接传入的配置对象，不需要重新加载
            # 重新加载基础权重参数
            self.synergy_level_weight = self.weights_config.get('base_weights', {}).get('synergy_level_weight', 1.0)
            self.synergy_count_weight = self.weights_config.get('base_weights', {}).get('synergy_count_weight', 0.5)
            self.chess_cost_weight = self.weights_config.get('base_weights', {}).get('chess_cost_weight', 0.1)
            
            # 重新加载自定义权重
            self.synergy_weights = self.weights_config.get('synergy_weights', {})
            self.chess_weights = self.weights_config.get('chess_weights', {})
            
            # 重新加载羁绊等级权重和棋子费用权重倍数
            synergy_level_weights = self.weights_config.get('synergy_level_weights', {})
            cost_weights = self.weights_config.get('cost_weights', {})
            
            # 将键转换为正确的类型
            self.level_multipliers = {int(k) if isinstance(k, str) else k: float(v) for k, v in synergy_level_weights.items()}
            self.cost_multipliers = {k: float(v) for k, v in cost_weights.items()}
        else:
            # 获取全局配置的最新实例
            config = get_global_weights_config()
            self.weights_config = config.config_data
            
            # 重新加载基础权重参数
            self.synergy_level_weight = config.get_base_weights().get('synergy_level_weight', 1.0)
            self.synergy_count_weight = config.get_base_weights().get('synergy_count_weight', 0.5)
            self.chess_cost_weight = config.get_base_weights().get('chess_cost_weight', 0.1)
            
            # 重新加载自定义权重
            self.synergy_weights = config.get_synergy_weights()
            self.chess_weights = config.get_chess_weights()
            
            # 重新加载羁绊等级权重和棋子费用权重倍数
            synergy_level_weights = config.get_synergy_level_weights()
            cost_weights = config.get_cost_weights()
            
            # 将键转换为正确的类型
            self.level_multipliers = {int(k) if isinstance(k, str) else k: float(v) for k, v in synergy_level_weights.items()}
            self.cost_multipliers = {k: float(v) for k, v in cost_weights.items()}
    
    def score_team(self, team) -> float:
        """
        对阵容进行综合评分
        
        Args:
            team: TeamComposition实例
            
        Returns:
            float: 评分结果
        """
        # 1. 羁绊等级评分
        synergy_level_score = self._score_synergy_levels(team)
        
        # 2. 羁绊种类数量评分
        synergy_count_score = self._score_synergy_count(team)
        
        # 3. 棋子费用评分
        chess_cost_score = self._score_chess_cost(team)
        
        # 4. 自定义羁绊和棋子权重评分
        custom_weight_score = self._score_custom_weights(team)
        
        # 5. 阵容整体评分
        # 综合以上四项，加权求和
        total_score = (
            synergy_level_score * self.synergy_level_weight +
            synergy_count_score * self.synergy_count_weight +
            chess_cost_score * self.chess_cost_weight +
            custom_weight_score  # 自定义权重分数已经包含各自的权重
        )
        
        return total_score
    
    def _score_synergy_levels(self, team) -> float:
        """
        评分羁绊等级
        
        Args:
            team: TeamComposition实例
            
        Returns:
            float: 羁绊等级评分
        """
        score = 0.0
        
        # 处理职业羁绊
        for job_level in team.synergy_levels.get('job', []):
            level = job_level['level']
            count = job_level['count']
            # 计算该羁绊的得分，根据等级和数量
            level_multiplier = self.level_multipliers.get(level, 1.0)
            # 额外激活的羁绊单位也计入评分
            extra_units = max(0, count - level)
            score += level * level_multiplier + extra_units * 0.1
        
        # 处理特质羁绊
        for race_level in team.synergy_levels.get('race', []):
            level = race_level['level']
            count = race_level['count']
            # 计算该羁绊的得分，根据等级和数量
            level_multiplier = self.level_multipliers.get(level, 1.0)
            # 额外激活的羁绊单位也计入评分
            extra_units = max(0, count - level)
            score += level * level_multiplier + extra_units * 0.1
        
        return score
    
    def _score_synergy_count(self, team) -> float:
        """
        评分激活的羁绊种类数量
        
        Args:
            team: TeamComposition实例
            
        Returns:
            float: 羁绊种类数量评分
        """
        # 获取激活的职业和特质数量
        active_job_count = len(team.synergy_levels.get('job', []))
        active_race_count = len(team.synergy_levels.get('race', []))
        
        # 根据激活的羁绊种类数量计算得分
        # 激活的羁绊种类越多，得分越高，但增长是非线性的
        return (active_job_count + active_race_count) ** 0.8
    
    def _score_chess_cost(self, team) -> float:
        """
        评分棋子费用分布
        
        Args:
            team: TeamComposition实例
            
        Returns:
            float: 棋子费用评分
        """
        score = 0.0
        
        # 根据棋子费用计算得分
        for chess in team.chess_list:
            cost = str(chess.get('price', '1'))
            # 费用越高的棋子，权重越大
            cost_multiplier = self.cost_multipliers.get(cost, 1.0)
            score += float(cost) * cost_multiplier
        
        return score
    
    def _score_custom_weights(self, team) -> float:
        """
        根据自定义权重对阵容进行评分
        
        Args:
            team: TeamComposition实例
            
        Returns:
            float: 自定义权重评分
        """
        score = 0.0
        
        # 1. 评分羁绊自定义权重
        for job_level in team.synergy_levels.get('job', []):
            synergy_name = job_level['name']
            if synergy_name in self.synergy_weights:
                weight = self.synergy_weights[synergy_name]
                score += weight * job_level['level']
        
        for race_level in team.synergy_levels.get('race', []):
            synergy_name = race_level['name']
            if synergy_name in self.synergy_weights:
                weight = self.synergy_weights[synergy_name]
                score += weight * race_level['level']
        
        # 2. 评分棋子自定义权重
        for chess in team.chess_list:
            chess_name = chess.get('displayName', '')
            if chess_name in self.chess_weights:
                weight = self.chess_weights[chess_name]
                score += weight
        
        return score
    
    def customize_scoring(self, **kwargs) -> None:
        """
        自定义评分参数
        
        Args:
            **kwargs: 参数名和值的映射
        """
        for key, value in kwargs.items():
            if hasattr(self, key):
                setattr(self, key, value)
                logger.info(f"自定义评分参数: {key} = {value}")
    
    def set_synergy_weight(self, synergy_name: str, weight: float) -> None:
        """
        设置特定羁绊的权重
        
        Args:
            synergy_name: 羁绊名称
            weight: 权重值
        """
        self.synergy_weights[synergy_name] = weight
        # 同时更新全局配置
        weights_config = get_global_weights_config()
        weights_config.set_synergy_weight(synergy_name, weight)
    
    def set_chess_weight(self, chess_name: str, weight: float) -> None:
        """
        设置特定棋子的权重
        
        Args:
            chess_name: 棋子名称
            weight: 权重值
        """
        self.chess_weights[chess_name] = weight
        # 同时更新全局配置
        weights_config = get_global_weights_config()
        weights_config.set_chess_weight(chess_name, weight)
    
    def set_synergy_level_weight(self, level: int, weight: float) -> None:
        """
        设置特定羁绊等级的权重
        
        Args:
            level: 羁绊等级
            weight: 权重值
        """
        self.level_multipliers[level] = weight
        # 同时更新全局配置
        weights_config = get_global_weights_config()
        weights_config.set_synergy_level_weight(str(level), weight)
    
    def set_cost_weight(self, cost: str, weight: float) -> None:
        """
        设置特定棋子费用的权重
        
        Args:
            cost: 棋子费用
            weight: 权重值
        """
        self.cost_multipliers[cost] = weight
        # 同时更新全局配置
        weights_config = get_global_weights_config()
        weights_config.set_cost_weight(cost, weight)
    
    def set_level_multiplier(self, level: int, multiplier: float) -> None:
        """
        设置特定羁绊等级的权重倍数
        
        Args:
            level: 羁绊等级
            multiplier: 权重倍数
        """
        self.level_multipliers[level] = multiplier
        # 同时更新全局配置
        weights_config = get_global_weights_config()
        weights_config.set_level_multiplier(level, multiplier)
    
    def set_cost_multiplier(self, cost: str, multiplier: float) -> None:
        """
        设置特定棋子费用的权重倍数
        
        Args:
            cost: 棋子费用
            multiplier: 权重倍数
        """
        self.cost_multipliers[cost] = multiplier
        # 同时更新全局配置
        weights_config = get_global_weights_config()
        weights_config.set_cost_multiplier(cost, multiplier)
    
    def set_synergy_level_multiplier(self, level: int, multiplier: float) -> None:
        """
        设置特定羁绊等级权重倍数
        
        Args:
            level: 羁绊等级
            multiplier: 权重倍数
        """
        self.level_multipliers[level] = multiplier
        # 同时更新全局配置
        weights_config = get_global_weights_config()
        weights_config.set_synergy_level_multiplier(level, multiplier)
    
    def set_cost_level_multiplier(self, level: int, multiplier: float) -> None:
        """
        设置特定羁绊等级费用权重倍数
        
        Args:
            level: 羁绊等级
            multiplier: 权重倍数
        """
        self.level_multipliers[level] = multiplier
        # 同时更新全局配置
        weights_config = get_global_weights_config()
        weights_config.set_cost_level_multiplier(level, multiplier) 