"""
云顶之弈阵容推荐器 - Web应用
"""
import os
import json
import logging
import copy
from flask import Flask, render_template, request, jsonify
from flask_cors import CORS
from waitress import serve
from typing import Optional, Dict, Any

from src.data_provider import DataQueryAPI
from src.recommendation import RecommendationEngine
from src.scoring.scoring_system import TeamScorer
from src.config import get_global_weights_config, WeightsConfig

# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger("TFT-Strategist-Web")

def create_app(config_path: Optional[str] = None):
    """
    创建Flask应用实例
    
    Args:
        config_path: 配置文件路径，用于全局权重配置
    """
    app = Flask(__name__, 
                static_folder='static',
                template_folder='templates')
    
    # 启用CORS，允许跨域请求
    CORS(app)
    
    # 加载全局权重配置
    weights_config = get_global_weights_config(config_path)
    
    # 加载数据和组件
    data_api = DataQueryAPI()
    scorer = TeamScorer(config_path=config_path)
    engine = RecommendationEngine(api=data_api, scorer=scorer, config_path=config_path)
    
    # 原始默认配置，用于重置操作
    default_config = copy.deepcopy(weights_config.config_data)
    
    @app.route('/')
    def index():
        """首页"""
        # 获取所有羁绊和棋子信息，用于前端展示
        all_jobs = data_api.get_all_jobs()
        all_races = data_api.get_all_races()
        all_synergies = all_jobs + all_races
        all_chess = data_api.get_all_chess()
        
        # 打印羁绊和棋子数据结构，用于调试
        logger.info(f"羁绊数量: {len(all_synergies)}")
        if all_synergies:
            sample_synergy = all_synergies[0]
            logger.info(f"羁绊示例: {sample_synergy}")
            
        logger.info(f"棋子数量: {len(all_chess)}")
        if all_chess:
            sample_chess = all_chess[0]
            logger.info(f"棋子示例: {sample_chess}")
        
        # 获取全局配置中的权重信息
        base_weights = weights_config.get_base_weights()
        synergy_weights = weights_config.get_synergy_weights()
        chess_weights = weights_config.get_chess_weights()
        
        # 打印权重配置，用于调试
        logger.info(f"羁绊权重示例: {list(synergy_weights.items())[:3]}")
        logger.info(f"棋子权重示例: {list(chess_weights.items())[:3]}")
        
        return render_template('index.html', 
                              synergies=all_synergies,
                              chess=all_chess,
                              base_weights=base_weights,
                              synergy_weights=synergy_weights,
                              chess_weights=chess_weights)
    
    @app.route('/api/recommend', methods=['POST'])
    def recommend():
        """阵容推荐API"""
        try:
            # 获取用户提交的参数
            data = request.json
            population = data.get('population', 9)
            num_results = data.get('num_results', 3)
            
            # 获取用户配置的权重
            base_weights = data.get('base_weights', {})
            synergy_weights = data.get('synergy_weights', {})
            chess_weights = data.get('chess_weights', {})
            
            # 创建临时配置对象，不修改全局配置
            temp_config = create_temp_config(
                weights_config.config_data,
                base_weights,
                synergy_weights,
                chess_weights
            )
            
            # 创建临时评分器
            temp_scorer = TeamScorer(config_obj=temp_config)
            
            # 创建临时推荐引擎
            temp_engine = RecommendationEngine(api=data_api, scorer=temp_scorer, config_obj=temp_config)
            
            # 获取必选棋子和必选羁绊
            required_chess_ids = data.get('required_chess', [])
            required_synergy_ids = data.get('required_synergies', [])
            
            # 将ID转换为字典格式，确保将ID转换为字符串
            required_chess = [{'id': str(chess_id)} for chess_id in required_chess_ids]
            required_synergies = [{'id': str(synergy_id)} for synergy_id in required_synergy_ids]
            
            # 调用推荐引擎生成阵容
            teams = temp_engine.recommend_team(
                population=population,
                required_synergies=required_synergies,
                required_chess=required_chess,
                max_results=num_results
            )
            
            # 将结果转换为JSON
            results = []
            for team in teams:
                # 获取阵容信息
                chess_list = []
                for chess_obj in team.chess_list:
                    chess_id = chess_obj.get('chessId')
                    if chess_id:
                        chess_list.append({
                            'id': chess_id,
                            'name': chess_obj.get('displayName', ''),
                            'cost': chess_obj.get('price', 0),
                            'image': f"chess_{chess_id}.png"  # 假设有对应图片
                        })
                
                # 获取激活的羁绊
                active_synergies = {}
                
                # 合并job和race的激活羁绊
                for synergy_type in ['job', 'race']:
                    for synergy_info in team.synergy_levels.get(synergy_type, []):
                        synergy_id = synergy_info.get('id')
                        if synergy_id:
                            # 取最高等级
                            if synergy_id not in active_synergies or active_synergies[synergy_id] < synergy_info.get('level', 0):
                                active_synergies[synergy_id] = synergy_info.get('level', 0)
                
                # 转换为前端需要的格式
                active_synergies_list = []
                for synergy_id, level in active_synergies.items():
                    synergy_info = data_api.get_synergy_by_id(synergy_id)
                    if synergy_info:
                        name = synergy_info.get('name', '')
                        active_synergies_list.append({
                            'id': synergy_id,
                            'name': name,
                            'level': level,
                            'description': synergy_info.get('description', ''),
                            'image': f"synergy_{synergy_id}.png"  # 假设有对应图片
                        })
                
                results.append({
                    'chess_list': chess_list,
                    'active_synergies': active_synergies_list,
                    'score': team.score
                })
            
            return jsonify({
                'status': 'success',
                'results': results
            })
            
        except Exception as e:
            logger.exception("推荐过程中发生错误")
            return jsonify({
                'status': 'error',
                'message': str(e)
            }), 500
    
    @app.route('/api/weights', methods=['GET'])
    def get_weights():
        """获取当前权重配置API"""
        return jsonify({
            'base_weights': weights_config.get_base_weights(),
            'synergy_weights': weights_config.get_synergy_weights(),
            'chess_weights': weights_config.get_chess_weights(),
            'synergy_level_weights': weights_config.get_synergy_level_weights(),
            'cost_weights': weights_config.get_cost_weights()
        })
    
    @app.route('/api/weights', methods=['POST'])
    def update_weights():
        """更新权重配置API"""
        try:
            data = request.json
            weights_config.update_config_data(data)
            return jsonify({'status': 'success'})
        except Exception as e:
            logger.exception("更新权重配置失败")
            return jsonify({
                'status': 'error',
                'message': str(e)
            }), 500
    
    @app.route('/api/details', methods=['GET'])
    def get_details():
        """获取棋子或羁绊的详细信息API"""
        try:
            item_type = request.args.get('type')  # 'chess' 或 'synergy'
            item_id = request.args.get('id')
            
            if not item_type or not item_id:
                return jsonify({
                    'status': 'error',
                    'message': '缺少必要参数'
                }), 400
            
            # 获取详细信息
            if item_type == 'chess':
                # 获取棋子详情
                chess = data_api.get_chess_by_id(item_id)
                if not chess:
                    return jsonify({
                        'status': 'error',
                        'message': f'未找到ID为{item_id}的棋子'
                    }), 404
                
                # 获取棋子的所有羁绊
                synergies = data_api.get_synergies_of_chess(item_id)
                
                # 构建详细信息
                details = {
                    'id': chess.get('chessId'),
                    'name': chess.get('displayName', ''),
                    'cost': chess.get('price', 0),
                    'skill_name': chess.get('skillName', ''),
                    'skill_description': chess.get('skillDetail', ''),
                    'skill_introduce': chess.get('skillIntroduce', ''),
                    'skill_type': chess.get('skillType', ''),
                    'skill_mana': chess.get('magic', ''),
                    # 添加更多棋子属性 - 使用与 chess.json 一致的字段名
                    'life': chess.get('life', ''),
                    'attack': chess.get('attack', ''),
                    'attackSpeed': chess.get('attackSpeed', ''),
                    'attackRange': chess.get('attackRange', ''),
                    'armor': chess.get('armor', ''),
                    'spellBlock': chess.get('spellBlock', ''),
                    'magic': chess.get('magic', ''),
                    'startMagic': chess.get('startMagic', ''),
                    'crit': chess.get('crit', ''),
                    'attackMag': chess.get('attackMag', ''),
                    'synergies': [{
                        'id': synergy.get('jobId') or synergy.get('raceId'),
                        'name': synergy.get('name', ''),
                        'type': 'job' if 'jobId' in synergy else 'race'
                    } for synergy in synergies]
                }
                
                return jsonify({
                    'status': 'success',
                    'type': 'chess',
                    'details': details
                })
                
            elif item_type == 'synergy':
                # 获取羁绊详情
                synergy = data_api.get_synergy_by_id(item_id)
                if not synergy:
                    return jsonify({
                        'status': 'error',
                        'message': f'未找到ID为{item_id}的羁绊'
                    }), 404
                
                # 获取羁绊的所有等级信息
                synergy_levels = synergy.get('level', {})
                
                # 获取该羁绊的所有棋子
                related_chess = data_api.get_chess_by_synergy(item_id)
                
                # 构建详细信息
                details = {
                    'id': synergy.get('jobId') or synergy.get('raceId'),
                    'name': synergy.get('name', ''),
                    'description': synergy.get('description', ''),
                    'introduce': synergy.get('introduce', ''),
                    'type': 'job' if 'jobId' in synergy else 'race',
                    'levels': [{
                        'level': level,
                        'effect': effect
                    } for level, effect in synergy_levels.items()],
                    # 添加相关棋子
                    'related_chess': [{
                        'id': chess.get('chessId'),
                        'name': chess.get('displayName', ''),
                        'cost': chess.get('price', 0)
                    } for chess in related_chess]
                }
                
                return jsonify({
                    'status': 'success',
                    'type': 'synergy',
                    'details': details
                })
                
            else:
                return jsonify({
                    'status': 'error',
                    'message': f'不支持的类型: {item_type}'
                }), 400
            
        except Exception as e:
            logger.exception("获取详细信息失败")
            return jsonify({
                'status': 'error',
                'message': str(e)
            }), 500
    
    return app

def create_temp_config(base_config: Dict[str, Any], 
                      base_weights: Dict[str, float], 
                      synergy_weights: Dict[str, float], 
                      chess_weights: Dict[str, float]) -> Dict[str, Any]:
    """
    创建临时权重配置
    
    Args:
        base_config: 基础配置
        base_weights: 基础权重
        synergy_weights: 羁绊权重
        chess_weights: 棋子权重
        
    Returns:
        Dict[str, Any]: 临时配置
    """
    # 深拷贝基础配置，避免修改原始配置
    temp_config = copy.deepcopy(base_config)
    
    # 更新基础权重
    if 'base_weights' not in temp_config:
        temp_config['base_weights'] = {}
        
    for param, value in base_weights.items():
        temp_config['base_weights'][param] = value
    
    # 更新羁绊权重
    if 'synergy_weights' not in temp_config:
        temp_config['synergy_weights'] = {}
        
    for synergy_name, weight in synergy_weights.items():
        temp_config['synergy_weights'][synergy_name] = weight
    
    # 更新棋子权重
    if 'chess_weights' not in temp_config:
        temp_config['chess_weights'] = {}
        
    for chess_name, weight in chess_weights.items():
        temp_config['chess_weights'][chess_name] = weight
    
    return temp_config

def run_server(host='0.0.0.0', port=5000, dev_mode=False, config_path=None):
    """
    运行Web服务器
    
    Args:
        host: 服务器主机地址
        port: 服务器端口
        dev_mode: 是否启用开发模式
        config_path: 配置文件路径
    """
    app = create_app(config_path)
    logger.info(f"启动Web服务器: {'开发模式' if dev_mode else '生产模式'}")
    
    if dev_mode:
        # 开发模式，使用Flask内置服务器
        app.run(host=host, port=port, debug=True)
    else:
        # 生产模式，使用waitress
        serve(app, host=host, port=port) 